/**************************************************************************

   Fotoxx      edit photos and manage collections

   Copyright 2007-2015 Michael Cornelison
   Source URL: http://kornelix.com/fotoxx
   Contact: kornelix@posteo.de

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see http://www.gnu.org/licenses/.

***************************************************************************

   Fotoxx image edit - Slide Show menu function

***************************************************************************/

#define EX extern                                                          //  enable extern declarations
#include "fotoxx.h"                                                        //  (variables in fotoxx.h are refs)
#include <sys/wait.h>

/**************************************************************************/

int   slideshow_dialog_event(zdialog *zd, cchar *event);                   //  user dialogs
void  ss_transprefs_dialog();                                              //  edit transition preferences
void  ss_imageprefs_dialog();                                              //  edit image preferences
void  ss_loadprefs();                                                      //  load preferences from file
void  ss_saveprefs();                                                      //  write preferences to file
void  ss_conv_prefsfile();                                                 //  convert prefs file to new format   v.15.02
int   ss_timerfunc(void *);                                                //  timer function
int   ss_nextrans();                                                       //  select next transition to use
void  ss_blankwindow();                                                    //  blank the window
void  ss_showcapcom(int mode);                                             //  show captions and comments

PIXBUF *ss_loadpxb(char *file);                                            //  load image as pixbuf

void  ss_instant();                                                        //  transition functions
void  ss_fadein();
void  ss_rollright();
void  ss_rolldown();
void  ss_venetian();
void  ss_grate();
void  ss_rectangle();
void  ss_implode();
void  ss_explode();
void  ss_radar();
void  ss_japfan();
void  ss_jaws();
void  ss_ellipse();
void  ss_raindrops();
void  ss_doubledoor();
void  ss_rotate();
void  ss_zoomin();

char        ss_albumfile[300] = "";                                        //  slide show album file
char        *ss_albumname = 0;                                             //  album name (ss_albumfile tail)
int         ss_Nfiles = 0;                                                 //  album file count
int         ss_seconds = 0;                                                //  standard display time from user
int         ss_cliplimit = 10;                                             //  image clipping limit from user
char        ss_musicfile[500] = "none";                                    //  /directory.../musicfile.ogg
int         ss_random = 0;                                                 //  use random transitions option from user
int         ss_fullscreen = 1;                                             //  flag, full screen mode
int         ss_replay = 0;                                                 //  flag, start over when done
int         ss_slowdown = 0;                                               //  slowdown factor for transition in use
float       ss_zoomsize = 1.0;                                             //  zoom-in size, 1-3.0 = 3x
int         ss_zoomsteps;                                                  //  zoom-in steps
int         ss_zoomlocx, ss_zoomlocy;                                      //  zoom-in target (50/50 = image midpoint)
int         ss_ww, ss_hh;                                                  //  slide show drawing window size
char        *ss_oldfile, *ss_newfile;                                      //  image files for transition
PIXBUF      *ss_pxbold, *ss_pxbnew;                                        //  pixbuf images: old, new
int         ss_rsold, ss_rsnew;                                            //  old and new pixbuf row stride
double      ss_timer = 0;                                                  //  slide show timer
cchar       *ss_state = 0;                                                 //  slide show state
int         ss_Fcurrent = 0;                                               //  next file to show
int         ss_Flast = -1;                                                 //  last file shown
int         ss_blank = 0;                                                  //  flag, user pressed B key (blank screen)
cairo_t     *ss_cr;                                                        //  cairo context

#define     SSNF 16                                                        //  slide show transition types
#define     SSMAXI 10000                                                   //  max. no. slide show images

struct ss_trantab_t {                                                      //  transition table
   char     tranname[32];                                                  //  transition name
   int      enabled;                                                       //  enabled or not
   int      slowdown;                                                      //  slowdown factor
   int      preference;                                                    //  relative preference, 0-99
   void     (*func)();                                                     //  function to perform transition
};

ss_trantab_t  ss_trantab[SSNF];                                            //  specific slide show transition preferences

int      ss_Tused[SSNF];                                                   //  list of transition types enabled
int      ss_Tlast[SSNF];                                                   //  last transitions used, in order
int      ss_Nused;                                                         //  count of enabled transitions, 0-SSNF
int      ss_Tnext;                                                         //  next transition to use >> last one used

ss_trantab_t  ss_trantab_default[SSNF] =                                   //  transition default preference
{    //   name           enab  slow  pref   function                       //  (enabled, slowdowns, equal preference)
      {  "instant",       1,    0,    10,   ss_instant     },              //  slowdowns appropriate for Intel integrated GPU
      {  "fade-in",       1,    5,    10,   ss_fadein      },
      {  "roll-right",    1,    3,    10,   ss_rollright   },              //  NO BLANKS IN TRANSITION NAMES
      {  "roll-down",     1,    3,    10,   ss_rolldown    },
      {  "venetian",      1,    1,    10,   ss_venetian    },
      {  "grate",         1,    1,    10,   ss_grate       },
      {  "rectangle",     1,    2,    10,   ss_rectangle   },
      {  "implode",       1,    8,    10,   ss_implode     },
      {  "explode",       1,    4,    10,   ss_explode     },
      {  "radar",         1,    1,    10,   ss_radar       },
      {  "Japanese-fan",  1,    4,    10,   ss_japfan      },
      {  "jaws",          1,    6,    10,   ss_jaws        },
      {  "ellipse",       1,   10,    10,   ss_ellipse     },
      {  "raindrops",     1,    3,    10,   ss_raindrops   },
      {  "doubledoor",    1,    8,    10,   ss_doubledoor  },
      {  "rotate",        1,    5,    10,   ss_rotate      }
};

struct ss_imagetab_t   {                                                   //  image table
   char     *imagefile;                                                    //  image file
   int      tone;                                                          //  flag, play tone when shown
   int      capsecs, commsecs;                                             //  seconds to show caption, comments
   int      wait1;                                                         //  seconds to wait
   float    zoomsize;                                                      //  zoom-in size, 1-3.0 = 3x
   int      zoomsteps;                                                     //  zoom-in steps
   int      zoomlocx, zoomlocy;                                            //  zoom-in target (50/50 = image midpoint)
   int      wait2;                                                         //  seconds to wait
   char     tranname[32];                                                  //  transition type to use
};

ss_imagetab_t  ss_imagetab[SSMAXI];                                        //  specific slide show image table


//  menu function - start or stop a slide show

void m_slideshow(GtkWidget *, cchar *)                                     //  overhauled                      v.14.04
{
   zdialog     *zd;
   int         zstat, ii;
   char        countmess[50], *pp;
   cchar       *esc_message = ZTX("Press ESC to exit slide show");

   strncpy0(ss_trantab_default[0].tranname,ZTX("instant"),32);             //  get translations into the .po file
   strncpy0(ss_trantab_default[1].tranname,ZTX("fade-in"),32);
   strncpy0(ss_trantab_default[2].tranname,ZTX("roll-right"),32);
   strncpy0(ss_trantab_default[3].tranname,ZTX("roll-down"),32);
   strncpy0(ss_trantab_default[4].tranname,ZTX("venetian"),32);
   strncpy0(ss_trantab_default[5].tranname,ZTX("grate"),32);
   strncpy0(ss_trantab_default[6].tranname,ZTX("rectangle"),32);
   strncpy0(ss_trantab_default[7].tranname,ZTX("implode"),32);
   strncpy0(ss_trantab_default[8].tranname,ZTX("explode"),32);
   strncpy0(ss_trantab_default[9].tranname,ZTX("radar"),32);
   strncpy0(ss_trantab_default[10].tranname,ZTX("Japanese-fan"),32);
   strncpy0(ss_trantab_default[11].tranname,ZTX("jaws"),32);
   strncpy0(ss_trantab_default[12].tranname,ZTX("ellipse"),32);
   strncpy0(ss_trantab_default[13].tranname,ZTX("raindrops"),32);
   strncpy0(ss_trantab_default[14].tranname,ZTX("doubledoor"),32);
   strncpy0(ss_trantab_default[15].tranname,ZTX("rotate"),32);

   F1_help_topic = "slide_show";
   if (checkpend("all")) return;                                           //  check nothing pending
   Fblock = 1;

/***
       _____________________________________________________
      | [x] [-] [ ]   Slide Show                            |
      |                                                     |
      | [Select] album-name   123 images                    |
      | Seconds [___|-+]    Clip Limit (%) [___|-+]         |
      | Music File: /directory/.../filename.ogg    [Browse] |
      | [x] Full Screen  [x] Auto-replay                    |
      | Press ESC to exit slide show                        |
      | Customize: [transitions]  [image files]             |
      |                                                     |
      |                                 [Proceed] [Cancel]  |
      |_____________________________________________________|

***/

   zd = zdialog_new(ZTX("Slide Show"),Mwin,Bproceed,Bcancel,null);         //  user dialog

   zdialog_add_widget(zd,"hbox","hbss","dialog",0,"space=3");
   zdialog_add_widget(zd,"button","selectalbum","hbss",Bselect,"space=5");
   zdialog_add_widget(zd,"label","albumname","hbss",Bnoselection,"space=5");
   zdialog_add_widget(zd,"label","nfiles","hbss",Bnoimages,"space=5");

   zdialog_add_widget(zd,"hbox","hbprefs","dialog",0,"space=2");
   zdialog_add_widget(zd,"label","labsecs","hbprefs",Bseconds,"space=5");
   zdialog_add_widget(zd,"spin","seconds","hbprefs","1|99|1|3");
   zdialog_add_widget(zd,"label","space","hbprefs",0,"space=5");
   zdialog_add_widget(zd,"label","labclip","hbprefs",ZTX("Clip Limit"),"space=5");
   zdialog_add_widget(zd,"spin","cliplim","hbprefs","0|50|1|0");

   zdialog_add_widget(zd,"hbox","hbmuf","dialog",0,"space=2");
   zdialog_add_widget(zd,"label","labmf","hbmuf",ZTX("Music File"),"space=3");
   zdialog_add_widget(zd,"entry","musicfile","hbmuf","none","size=40|space=5");
   zdialog_add_widget(zd,"button","browse","hbmuf",Bbrowse,"space=5");

   zdialog_add_widget(zd,"hbox","hbscreen","dialog",0,"space=2");
   zdialog_add_widget(zd,"check","fullscreen","hbscreen",ZTX("Full Screen"),"space=3");
   zdialog_add_widget(zd,"check","replay","hbscreen",ZTX("Auto-replay"),"space=3");

   zdialog_add_widget(zd,"hbox","hbesc","dialog",0,"space=5");
   zdialog_add_widget(zd,"label","labesc","hbesc",esc_message,"space=20");

   zdialog_add_widget(zd,"hbox","hbcust","dialog",0,"space=5");
   zdialog_add_widget(zd,"label","labprefs","hbcust",ZTX("Customize:"),"space=5");
   zdialog_add_widget(zd,"button","transprefs","hbcust",ZTX("transitions"),"space=5");
   zdialog_add_widget(zd,"button","imageprefs","hbcust",ZTX("image files"),"space=5");

   if (ss_Nfiles) {                                                        //  if album file available,
      ss_loadprefs();                                                      //    load slide show data or defaults
      zdialog_stuff(zd,"albumname",ss_albumname);
      snprintf(countmess,50,ZTX("%d images"),ss_Nfiles);
      zdialog_stuff(zd,"nfiles",countmess);
      zdialog_stuff(zd,"seconds",ss_seconds);
      zdialog_stuff(zd,"cliplim",ss_cliplimit);
      zdialog_stuff(zd,"musicfile",ss_musicfile);
      zdialog_stuff(zd,"fullscreen",ss_fullscreen);                        //  v.15.03
      zdialog_stuff(zd,"replay",ss_replay);
      navi::gallerytype = 4;                                               //  open gallery with slide show album
      gallery(ss_albumfile,"initF");
      gallery(0,"paint",ss_Fcurrent);
      m_viewmode(0,"G");
   }

   zdialog_run(zd,slideshow_dialog_event,"save");                          //  run dialog
   zstat = zdialog_wait(zd);                                               //  wait for completion

   if (zstat != 1) {                                                       //  cancel
      zdialog_free(zd);
      Fblock = 0;
      return;
   }

   if (! ss_Nfiles) {                                                      //  no selection
      zmessageACK(Mwin,0,ZTX("invalid album"));
      zdialog_free(zd);
      Fblock = 0;
      return;
   }

   zdialog_fetch(zd,"seconds",ss_seconds);                                 //  timer interval, seconds
   if (ss_Nused == 0) ss_seconds = 9999;                                   //  if only arrow-keys used, huge interval
   zdialog_fetch(zd,"cliplim",ss_cliplimit);                               //  image clipping limit
   zdialog_fetch(zd,"musicfile",ss_musicfile,500);                         //  music file
   zdialog_fetch(zd,"fullscreen",ss_fullscreen);                           //  full screen option
   zdialog_fetch(zd,"replay",ss_replay);                                   //  replay option (last --> first image)

   zdialog_free(zd);                                                       //  kill dialog

   ss_saveprefs();                                                         //  save preference changes

   if (curr_file) {                                                        //  start at curr. file
      for (ii = 0; ii < ss_Nfiles; ii++)                                   //    if member of file list
         if (strmatch(curr_file,ss_imagetab[ii].imagefile)) break;
      if (ii == ss_Nfiles) ii = 0;
   }
   else ii = 0;
   ss_Fcurrent = ii;                                                       //  next file in list to show

   m_viewmode(0,"F");                                                      //  insure tab F

   if (ss_fullscreen) win_fullscreen(1);                                   //  full screen, hide menu and panel

   ss_newfile = 0;                                                         //  no new image
   ss_pxbnew = 0;
   ss_oldfile = 0;                                                         //  no old (prior) image
   ss_pxbold = 0;

   Fslideshow = 1;                                                         //  slideshow active for KB events
   ss_blank = 0;                                                           //  not blank window
   ss_state = "first";
   Fblowup = 1;                                                            //  expand small images
   Fzoom = 0;                                                              //  fit window

   pp = ss_musicfile;                                                      //  start music if any
   if (*pp == '/') shell_ack("paplay \"%s\" &",pp);

   g_timeout_add(100,ss_timerfunc,0);
   return;
}


//  dialog event function - file chooser for images to show and music file

int slideshow_dialog_event(zdialog *zd, cchar *event)
{
   char        *file, *pp;
   char        countmess[50];

   if (strmatch(event,"focus"))
      F1_help_topic = "slide_show";

   if (strmatch(event,"enter")) zd->zstat = 1;                             //  [proceed]

   if (zd->zstat == 1) {                                                   //  [proceed]
      if (ss_Nfiles) return 1;
      zmessageACK(Mwin,0,ZTX("invalid album"));                            //  diagnose and keep dialog open      v.15.01
      zd->zstat = 0;
      return 1;
   }

   if (zd->zstat) return 1;                                                //  cancel or [x]

   if (strmatch(event,"selectalbum"))                                      //  select a slide show album
   {
      ss_Nfiles = 0;                                                       //  reset album data
      ss_albumfile[0] = 0;
      zdialog_stuff(zd,"albumname",Bnoselection);
      zdialog_stuff(zd,"nfiles",Bnoimages);

      file = zgetfile(ZTX("open album"),MWIN,"file",albums_dirk);
      if (! file) return 1;
      if (strlen(file) > 299) {
         zmessageACK(Mwin,0,"file name too long");
         return 1;
      }

      strcpy(ss_albumfile,file);
      pp = strrchr(ss_albumfile,'/');                                      //  get album name
      ss_albumname = pp + 1;
      zfree(file);

      ss_loadprefs();                                                      //  get slide show prefs or defaults
      if (! ss_Nfiles) return 1;

      zdialog_stuff(zd,"albumname",ss_albumname);                          //  update dialog album data
      snprintf(countmess,50,ZTX("%d images"),ss_Nfiles);
      zdialog_stuff(zd,"nfiles",countmess);
      zdialog_stuff(zd,"seconds",ss_seconds);
      zdialog_stuff(zd,"cliplim",ss_cliplimit);
      zdialog_stuff(zd,"musicfile",ss_musicfile);
      zdialog_stuff(zd,"fullscreen",ss_fullscreen);                        //  v.15.03
      zdialog_stuff(zd,"replay",ss_replay);

      navi::gallerytype = 4;                                               //  open gallery with slide show album
      gallery(ss_albumfile,"initF");
      m_viewmode(0,"G");
      gallery(0,"paint",0);
   }

   if (strmatch(event,"browse")) {                                         //  browse for music file
      pp = ss_musicfile;
      pp = zgetfile(ZTX("Select music file"),MWIN,"file",pp);
      if (! pp) pp = zstrdup("none");
      zdialog_stuff(zd,"musicfile",pp);
      zfree(pp);
   }

   if (! ss_Nfiles) return 1;

   if (strmatch(event,"transprefs")) ss_transprefs_dialog();               //  edit transition preferences
   if (strmatch(event,"imageprefs")) ss_imageprefs_dialog();               //  edit image preferences

   return 1;
}


//  set transitions preferences for specific slide show

void ss_transprefs_dialog()
{
   zdialog     *zd;
   int         ii, jj;
   char        nameii[SSNF], enabii[SSNF], slowii[SSNF], prefii[SSNF];
   cchar       *randmess = ZTX("select random (if 5+ enabled)");

/***
          _______________________________________________
         |        Transition Preferences                 |
         |                                               |
         |  [x] select random (if 5+ enabled)            |
         |                                               |
         |  transition    enabled  slowdown  preference  |
         |  instant         [x]     [ 1 ]      [ 10 ]    |
         |  fade-in         [x]     [ 2 ]      [ 20 ]    |
         |  roll-right      [x]     [ 3 ]      [  0 ]    |
         |  ....            ...     ...        ...       |
         |                                               |
         |                                       [done]  |
         |_______________________________________________|

***/

   if (! ss_Nfiles) {
      zmessageACK(Mwin,0,ZTX("invalid album"));
      return;
   }

   zd = zdialog_new(ZTX("Transition Preferences"),Mwin,Bdone,null);

   zdialog_add_widget(zd,"hbox","hbrand","dialog",0,"space=3");
   zdialog_add_widget(zd,"check","rand","hbrand",randmess,"space=3");
   zdialog_add_widget(zd,"hbox","hb1","dialog",0,"space=3");
   zdialog_add_widget(zd,"vbox","vb1","hb1",0,"space=3|homog");
   zdialog_add_widget(zd,"vbox","vb2","hb1",0,"space=3|homog");
   zdialog_add_widget(zd,"vbox","vb3","hb1",0,"space=3|homog");
   zdialog_add_widget(zd,"vbox","vb4","hb1",0,"space=3|homog");
   zdialog_add_widget(zd,"label","labname","vb1",ZTX("transition"));
   zdialog_add_widget(zd,"label","labenab","vb2",ZTX("enabled"));
   zdialog_add_widget(zd,"label","labslow","vb3",ZTX("slowdown"));
   zdialog_add_widget(zd,"label","labpref","vb4",ZTX("preference"));

   zdialog_stuff(zd,"rand",ss_random);                                     //  stuff random checkbox

   for (ii = 0; ii < SSNF; ii++) {                                         //  build input dialog for transition prefs
      snprintf(nameii,SSNF,"name_%d",ii);
      snprintf(enabii,SSNF,"enab_%d",ii);
      snprintf(slowii,SSNF,"slow_%d",ii);
      snprintf(prefii,SSNF,"pref_%d",ii);
      zdialog_add_widget(zd,"label",nameii,"vb1","transition");
      zdialog_add_widget(zd,"check",enabii,"vb2",0,"size=3");
      zdialog_add_widget(zd,"spin",slowii,"vb3","0|99|1|1","size=3");
      zdialog_add_widget(zd,"spin",prefii,"vb4","0|99|1|10","size=3");
      zdialog_stuff(zd,nameii,ss_trantab[ii].tranname);                    //  stuff current transition prefs
      zdialog_stuff(zd,enabii,ss_trantab[ii].enabled);
      zdialog_stuff(zd,slowii,ss_trantab[ii].slowdown);
      zdialog_stuff(zd,prefii,ss_trantab[ii].preference);
   }

   zdialog_run(zd);                                                        //  run dialog, wait for completion
   zdialog_wait(zd);

   zdialog_fetch(zd,"rand",ss_random);                                     //  get mode, 0/1 = sequential/random

   for (ii = 0; ii < SSNF; ii++) {
      snprintf(enabii,SSNF,"enab_%d",ii);
      snprintf(slowii,SSNF,"slow_%d",ii);
      snprintf(prefii,SSNF,"pref_%d",ii);
      zdialog_fetch(zd,enabii,ss_trantab[ii].enabled);
      zdialog_fetch(zd,slowii,ss_trantab[ii].slowdown);
      zdialog_fetch(zd,prefii,ss_trantab[ii].preference);
   }

   zdialog_free(zd);

   ss_saveprefs();

   for (ii = jj = 0; ii < SSNF; ii++) {                                    //  initialize list of enabled
      if (ss_trantab[ii].enabled) {                                        //    and last used transition types
         ss_Tused[jj] = ii;
         jj++;
      }
      ss_Tlast[ii] = 0;
   }

   ss_Nused = jj;                                                          //  no. enabled transition types
   ss_Tnext = 0;                                                           //  next one to use (first)

   return;
}


//  set image preferences for specific slide show

void ss_imageprefs_dialog()                                                //  v.14.04
{
   int ss_imageprefs_dialog_event(zdialog *zd, cchar *event);

   zdialog  *zd;
   int      ii;
   char     *pp;

/***
       _______________________________________________
      |        Image Preferences                      |
      |                                               |
      |  Image File: /path.../filename.jpg            |
      |  Play Tone [x]                                |
      |  Show Caption  [  0 ] Seconds                 |
      |  Show Comments [ 12 ] Seconds                 |
      |  Wait [ 3 ] Seconds                           |
      |  Zoom size (x) [ 2.5 ]   Steps [ 200 ]        |
      |  Zoom location  width [40]  height [60]       |
      |  Wait [ 6 ] Seconds                           |
      |  Transition [ rotate |v]                      |
      |                                               |
      |                                       [Done]  |
      |_______________________________________________|

***/

   if (! ss_Nfiles) {
      zmessageACK(Mwin,0,ZTX("invalid album"));
      return;
   }

   m_viewmode(0,"G");                                                      //  gallery view                       v.15.01

   zd = zdialog_new(ZTX("Image Preferences"),Mwin,Bdone,null);
   zd_ss_imageprefs = zd;

   zdialog_add_widget(zd,"hbox","hbimf","dialog",0,"space=5");
   zdialog_add_widget(zd,"label","labimf","hbimf",ZTX("Image File:"),"space=3");
   zdialog_add_widget(zd,"label","imagefile","hbimf",0,"space=3");

   zdialog_add_widget(zd,"hbox","hbpt","dialog");
   zdialog_add_widget(zd,"check","tone","hbpt",ZTX("Play tone"),"space=3");

   zdialog_add_widget(zd,"hbox","hbcap","dialog");
   zdialog_add_widget(zd,"label","labcap","hbcap",ZTX("Show Caption"),"space=3");
   zdialog_add_widget(zd,"spin","caption","hbcap","0|99|1|0","space=3");
   zdialog_add_widget(zd,"label","labsecs","hbcap",Bseconds,"space=5");

   zdialog_add_widget(zd,"hbox","hbcom","dialog");
   zdialog_add_widget(zd,"label","labcom","hbcom",ZTX("Show Comments"),"space=3");
   zdialog_add_widget(zd,"spin","comments","hbcom","0|99|1|0","space=3");
   zdialog_add_widget(zd,"label","labsecs","hbcom",Bseconds,"space=5");

   zdialog_add_widget(zd,"hbox","hbbz","dialog");
   zdialog_add_widget(zd,"label","labwait","hbbz",ZTX("Wait"),"space=3");
   zdialog_add_widget(zd,"spin","wait1","hbbz","0|99|1|0","space=3");
   zdialog_add_widget(zd,"label","labbzs2","hbbz",Bseconds,"space=5");

   zdialog_add_widget(zd,"hbox","hbz","dialog");
   zdialog_add_widget(zd,"label","labzsz","hbz",ZTX("Zoom size (x)"),"space=3");
   zdialog_add_widget(zd,"spin","zoomsize","hbz","1.0|3.0|0.1|1.0","space=3");
   zdialog_add_widget(zd,"label","space","hbz",0,"space=5");
   zdialog_add_widget(zd,"label","labzst","hbz",ZTX("Steps"),"space=3");
   zdialog_add_widget(zd,"spin","zoomsteps","hbz","50|999|1|200");

   zdialog_add_widget(zd,"hbox","hbzloc","dialog");
   zdialog_add_widget(zd,"label","labzloc","hbzloc",ZTX("Zoom location"),"space=3");
   zdialog_add_widget(zd,"label","labzlocx","hbzloc",Bwidth,"space=8");
   zdialog_add_widget(zd,"label","zoomlocx","hbzloc"," 50 ");
   zdialog_add_widget(zd,"label","space","hbzloc",0,"space=5");
   zdialog_add_widget(zd,"label","labzlocy","hbzloc",Bheight,"space=8");
   zdialog_add_widget(zd,"label","zoomlocy","hbzloc"," 50 ");

   zdialog_add_widget(zd,"hbox","hbaz","dialog");
   zdialog_add_widget(zd,"label","labwait","hbaz",ZTX("Wait"),"space=3");
   zdialog_add_widget(zd,"spin","wait2","hbaz","0|99|1|0","space=3");
   zdialog_add_widget(zd,"label","labbzs2","hbaz",Bseconds,"space=5");

   zdialog_add_widget(zd,"hbox","hbtrn","dialog");
   zdialog_add_widget(zd,"label","labtr","hbtrn",ZTX("Transition"),"space=3");
   zdialog_add_widget(zd,"combo","tranname","hbtrn");

   zdialog_cb_app(zd,"tranname",ZTX("next"));                              //  default transition

   for (ii = 0; ii < SSNF; ii++)                                           //  add all transitions to dropdown list
      zdialog_cb_app(zd,"tranname",ss_trantab[ii].tranname);

   if (curr_file) {                                                        //  set to curr. file
      for (ii = 0; ii < ss_Nfiles; ii++)                                   //    if member of album
         if (strmatch(curr_file,ss_imagetab[ii].imagefile)) break;
      if (ii == ss_Nfiles) ii = 0;
   }
   else ii = 0;
   ss_Fcurrent = ii;

   pp = strrchr(ss_imagetab[ii].imagefile,'/');                            //  stuff curr. image file data
   if (pp) zdialog_stuff(zd,"imagefile",pp+1);                             //    into dialog
   zdialog_stuff(zd,"tone",ss_imagetab[ii].tone);
   zdialog_stuff(zd,"caption",ss_imagetab[ii].capsecs);
   zdialog_stuff(zd,"comments",ss_imagetab[ii].commsecs);
   zdialog_stuff(zd,"wait1",ss_imagetab[ii].wait1);
   zdialog_stuff(zd,"zoomsize",ss_imagetab[ii].zoomsize);
   zdialog_stuff(zd,"zoomsteps",ss_imagetab[ii].zoomsteps);
   zdialog_stuff(zd,"zoomlocx",ss_imagetab[ii].zoomlocx);
   zdialog_stuff(zd,"zoomlocy",ss_imagetab[ii].zoomlocy);
   zdialog_stuff(zd,"wait2",ss_imagetab[ii].wait2);
   zdialog_stuff(zd,"tranname",ss_imagetab[ii].tranname);

   zdialog_run(zd,ss_imageprefs_dialog_event);
   zdialog_wait(zd);
   zdialog_free(zd);

   zd_ss_imageprefs = 0;
   ss_saveprefs();

   return;
}


//  image prefs dialog event and completion function

int  ss_imageprefs_dialog_event(zdialog *zd, cchar *event)
{
   int      ii, jj;
   float    ff;
   char     tranname[32];

   ii = ss_Fcurrent;                                                       //  from mouse click on thumbnail
   if (ii >= ss_Nfiles) return 1;

   if (strmatch(event,"tone")) {
      zdialog_fetch(zd,"tone",jj);
      ss_imagetab[ii].tone = jj;
   }

   if (strmatch(event,"caption")) {
      zdialog_fetch(zd,"caption",jj);
      ss_imagetab[ii].capsecs = jj;
   }

   if (strmatch(event,"comments")) {
      zdialog_fetch(zd,"comments",jj);
      ss_imagetab[ii].commsecs = jj;
   }

   if (strmatch(event,"wait1")) {
      zdialog_fetch(zd,"wait1",jj);
      ss_imagetab[ii].wait1 = jj;
   }

   if (strmatch(event,"zoomsize")) {
      zdialog_fetch(zd,"zoomsize",ff);
      ss_imagetab[ii].zoomsize = ff;
   }

   if (strmatch(event,"zoomsteps")) {
      zdialog_fetch(zd,"zoomsteps",jj);
      ss_imagetab[ii].zoomsteps = jj;
   }

   if (strmatch(event,"zoomlocx")) {
      zdialog_fetch(zd,"zoomlocx",jj);
      ss_imagetab[ii].zoomlocx = jj;
   }

   if (strmatch(event,"zoomlocy")) {
      zdialog_fetch(zd,"zoomlocy",jj);
      ss_imagetab[ii].zoomlocy = jj;
   }

   if (strmatch(event,"wait2")) {
      zdialog_fetch(zd,"wait2",jj);
      ss_imagetab[ii].wait2 = jj;
   }

   if (strmatch(event,"tranname")) {
      zdialog_fetch(zd,"tranname",tranname,32);
      if (! strmatch(tranname,"next")) {
         for (jj = 0; jj < SSNF; jj++)
            if (strmatch(tranname,ss_trantab[jj].tranname)) break;
         if (jj == SSNF) return 1;
      }
      strncpy0(ss_imagetab[ii].tranname,tranname,32);
   }

   return 1;
}


//  response function for gallery thumbnail left-click
//  stuff image prefs dialog with data for clicked image

void ss_imageprefs_Lclick_func(int Nth)                                    //  v.14.04
{
   zdialog     *zd;
   int         ii;
   char        *pp;

   if (! clicked_file) return;
   zfree(clicked_file);
   clicked_file = 0;

   zd = zd_ss_imageprefs;
   if (! zd) return;

   if (! ss_Nfiles) {
      zmessageACK(Mwin,0,ZTX("invalid album"));
      return;
   }

   ii = Nth;
   if (ii >= ss_Nfiles) return;
   ss_Fcurrent = ii;

   ss_imagetab[ii].zoomlocx = clicked_width;                               //  set zoom-in location from
   ss_imagetab[ii].zoomlocy = clicked_height;                              //    thumbnail click position

   pp = strrchr(ss_imagetab[ii].imagefile,'/');                            //  stuff image data into dialog
   if (pp) zdialog_stuff(zd,"imagefile",pp+1);
   zdialog_stuff(zd,"tone",ss_imagetab[ii].tone);
   zdialog_stuff(zd,"caption",ss_imagetab[ii].capsecs);
   zdialog_stuff(zd,"comments",ss_imagetab[ii].commsecs);
   zdialog_stuff(zd,"wait1",ss_imagetab[ii].wait1);
   zdialog_stuff(zd,"zoomsize",ss_imagetab[ii].zoomsize);
   zdialog_stuff(zd,"zoomsteps",ss_imagetab[ii].zoomsteps);
   zdialog_stuff(zd,"zoomlocx",ss_imagetab[ii].zoomlocx);
   zdialog_stuff(zd,"zoomlocy",ss_imagetab[ii].zoomlocy);
   zdialog_stuff(zd,"wait2",ss_imagetab[ii].wait2);
   zdialog_stuff(zd,"tranname",ss_imagetab[ii].tranname);

   return;
}


/******************  new preferences file format v.15.02  *****************

   global data:
   seconds: NN                      0-999
   cliplimit: NN                    0-50
   random: N                        0-1
   musicfile: /path.../file.ogg     music file or "none"
   fullscreen:                      0-1
   replay:                          0-1

   transitions data:
   tranname N NN NN                 tranname enabled slowdown preference
   tranname N NN NN                            0-1     0-99     0-99
   ...

   images data:
   imagefile: /path.../file.jpg
   tone: N                          0-1
   caption: NN                      seconds to show caption
   comments: NN                     seconds to show comments
   wait1: NN                        seconds to wait 0-99
   zoomsize: N.N                    1.0 - 3.0
   zoomsteps: NNN                   100-999
   zoomloc: NN NN                   20-80 20-80
   wait2: NN                        seconds to wait 0-99
   tranname: aaaaaa                 transition name or "next"
   ...

***************************************************************************/


//  Load all data for a specific slide show from a slide show preferences file.
//  Set defaults if no data previously defined.

void ss_loadprefs()                                                        //  v.14.04
{
   FILE        *fid;
   char        buff[XFCC];
   char        prefsfile[300], *pp;
   int         ii, jj, nn, ff, ftype, format;
   char        tranname[32];
   int         n1, n2, n3;

   for (ii = 0; ii < ss_Nfiles; ii++) {                                    //  free prior image data if any
      pp = ss_imagetab[ii].imagefile;
      if (pp) zfree(pp);
      ss_imagetab[ii].imagefile = 0;
   }

   ss_Nfiles = 0;

   fid = fopen(ss_albumfile,"r");                                          //  open album file
   if (! fid) {
      zmessageACK(Mwin,0,ZTX("invalid album"));
      return;
   }

   for (ii = 0; ii < SSMAXI; ) {                                           //  read all image file names
      pp = fgets_trim(buff,XFCC,fid);
      if (! pp) break;
      ftype = image_file_type(pp);                                         //  screen out deleted image files
      if (ftype != 2) continue;
      ss_imagetab[ii].imagefile = zstrdup(pp);                             //  add to image table
      ii++;
   }

   fclose(fid);
   ss_Nfiles = ii;

   if (! ss_Nfiles) {
      zmessageACK(Mwin,0,ZTX("invalid album"));
      return;
   }

   navi::gallerytype = 4;                                                  //  open gallery with slide show album
   gallery(ss_albumfile,"initF");
   m_viewmode(0,"G");
   gallery(0,"paint",0);

   ss_seconds = 3;                                                         //  defaults: image display time
   ss_cliplimit = 0;                                                       //  image clip limit = no clipping
   strcpy(ss_musicfile,"none");                                            //  no music file

   for (ii = 0; ii < SSNF; ii++)                                           //  initialize transitions table
      ss_trantab[ii] = ss_trantab_default[ii];                             //    with default preferences

   for (ii = 0; ii < SSNF; ii++) {
      ss_Tused[ii] = ii;                                                   //  all transition types are used
      ss_Tlast[ii] = 0;                                                    //  last used list is empty
   }

   ss_random = 0;                                                          //  random transitions = NO
   ss_Nused = SSNF;                                                        //  used transitions = all
   ss_Tnext = 0;                                                           //  next = first

   for (ii = 0; ii < ss_Nfiles; ii++) {                                    //  initialize image table with defaults
      ss_imagetab[ii].tone = 0;
      ss_imagetab[ii].capsecs = 0;
      ss_imagetab[ii].commsecs = 0;
      ss_imagetab[ii].wait1 = 0;
      ss_imagetab[ii].zoomsize = 1.0;                                      //  1.0 = no zoom-in
      ss_imagetab[ii].zoomsteps = 200;
      ss_imagetab[ii].zoomlocx = 50;
      ss_imagetab[ii].zoomlocy = 50;
      ss_imagetab[ii].wait2 = 0;
      strcpy(ss_imagetab[ii].tranname,"next");
   }

   snprintf(prefsfile,300,"%s/%s",slideshow_dirk,ss_albumname);
   fid = fopen(prefsfile,"r");                                             //  open slide show prefs file
   if (! fid) return;

   format = 0;

   while (true)
   {
      pp = fgets_trim(buff,XFCC,fid,1);
      if (! pp) break;

      if (strmatchN(pp,"overall:",8)) {                                    //  detect old file format
         fclose(fid);
         ss_conv_prefsfile();                                              //  convert to new format              v.15.02
         ss_loadprefs();                                                   //  process new format
         return;
      }

      if (strmatchN(pp,"global data:",12)) {
         format = 1;
         continue;
      }

      if (strmatchN(pp,"transitions data:",17)) {
         format = 2;
         continue;
      }

      if (strmatchN(pp,"images data:",12)) {
         format = 3;
         continue;
      }

      if (format == 1)                                                     //  overall preferences
      {
         if (strmatchN(pp,"seconds: ",9)) {                                //  seconds: NN   seconds to show each image
            ss_seconds = atoi(pp+9);
            continue;
         }

         if (strmatchN(pp,"cliplimit: ",11)) {                             //  cliplimit: NN   margin clip limit  0-50 %
            ss_cliplimit = atoi(pp+11);
            continue;
         }

         if (strmatchN(pp,"random: ",8)) {                                 //  random: N   0-1 = seq./random transactions
            ss_random = atoi(pp+8);
            continue;
         }

         if (strmatchN(pp,"musicfile: ",11)) {                             //  musicfile: /directory/.../musicfile.ogg
            strncpy0(ss_musicfile,pp+11,500);
            continue;
         }

         if (strmatchN(pp,"fullscreen: ",12)) {                            //  fullscreen: N   0-1 = no / full screen
            ss_fullscreen = atoi(pp+12);                                   //  v.15.03
            continue;
         }

         if (strmatchN(pp,"replay: ",8)) {                                 //  random: N   0-1 = no / replay after end
            ss_replay = atoi(pp+8);                                        //  v.15.03
            continue;
         }
      }

      if (format == 2)                                                     //  transition preferences
      {
         nn = sscanf(buff,"%s %d %d %d ",tranname,&n1,&n2,&n3);            //  tranname        N  NN  NN
         if (nn != 4) goto format_error;                                   //  (enabled 0-1  slowdown 0-99  pref. 0-99)
         for (ii = 0; ii < SSNF; ii++)
            if (strmatch(tranname,ss_trantab[ii].tranname)) break;
         if (ii == SSNF) goto format_error;
         ss_trantab[ii].enabled = n1;
         ss_trantab[ii].slowdown = n2;
         ss_trantab[ii].preference = n3;
      }

      if (format == 3)                                                     //  image file preferences
      {
         if (strmatchN(pp,"imagefile: ",11)) {                             //  set image file for subsequent recs
            pp += 11;
            if (*pp != '/') goto format_error;
            for (ii = 0; ii < ss_Nfiles; ii++)                             //  search album for matching image
               if (strmatch(pp,ss_imagetab[ii].imagefile)) break;
            if (ii == ss_Nfiles) ii = -1;                                  //  if not found, set no curr. image
            continue;
         }

         if (ii < 0) continue;                                             //  ignore recs following invalid image file

         if (strmatchN(pp,"tone: ",6)) {                                   //  tone: N    0 or 1 = play tone
            nn = atoi(pp+6);
            ss_imagetab[ii].tone = nn;
            continue;
         }

         if (strmatchN(pp,"caption: ",9)) {                                //  caption: NN   seconds to show caption
            nn = atoi(pp+9);
            ss_imagetab[ii].capsecs = nn;
            continue;
         }

         if (strmatchN(pp,"comments: ",10)) {                              //  comments: NN   seconds to show comments
            nn = atoi(pp+10);
            ss_imagetab[ii].commsecs = nn;
            continue;
         }

         if (strmatchN(pp,"wait1: ",7)) {                                  //  wait1: NN   seconds before zoom-in
            nn = atoi(pp+7);
            ss_imagetab[ii].wait1 = nn;
            continue;
         }

         if (strmatchN(pp,"zoomsize: ",10)) {                              //  zoomsize: N.N   1.0 - 3.0 = 3x     v.15.02
            ff = atof(pp+10);
            ss_imagetab[ii].zoomsize = ff;
            continue;
         }

         if (strmatchN(pp,"zoomsteps: ",11)) {                             //  zoomsteps: NNN   zoom-in steps 100-999
            nn = atoi(pp+11);
            ss_imagetab[ii].zoomsteps = nn;
            continue;
         }

         if (strmatchN(pp,"zoomloc: ",9)) {                                //  zoomloc: NN NN   zoom-in location
            nn = strtol(pp+9,&pp,10);                                      //  (20-80% of image width and height)
            ss_imagetab[ii].zoomlocx = nn;
            nn = atoi(pp);
            ss_imagetab[ii].zoomlocy = nn;
            continue;
         }

         if (strmatchN(pp,"wait2: ",7)) {                                  //  wait2: NN   seconds after zoom-in
            nn = atoi(pp+7);
            ss_imagetab[ii].wait2 = nn;
            continue;
         }

         if (strmatchN(pp,"tranname: ",10)) {                              //  transaction to next image
            strncpy0(ss_imagetab[ii].tranname,pp+10,32);
            continue;
         }
      }
   }

   fclose(fid);

   for (ii = jj = 0; ii < SSNF; ii++) {                                    //  initialize list of enabled
      if (ss_trantab[ii].enabled) {                                        //    transition types
         ss_Tused[jj] = ii;
         jj++;
      }
   }

   ss_Nused = jj;                                                          //  no. enabled transition types
   return;

format_error:
   zmessageACK(Mwin,0,ZTX("file format error: \n %s"),buff);
   fclose(fid);
   return;
}


//  Save all data for a specific slide show to a slide show preferences file.

void ss_saveprefs()                                                        //  v.14.04
{
   FILE        *fid;
   char        prefsfile[300];
   int         ii;

   if (! ss_Nfiles) {
      zmessageACK(Mwin,0,ZTX("invalid album"));
      return;
   }

   snprintf(prefsfile,300,"%s/%s",slideshow_dirk,ss_albumname);
   fid = fopen(prefsfile,"w");                                             //  open slide show prefs file
   if (! fid) {
      zmessageACK(Mwin,0,strerror(errno));
      return;
   }

   fprintf(fid,"global data: \n");
   fprintf(fid,"seconds: %d \n",ss_seconds);
   fprintf(fid,"cliplimit: %d \n",ss_cliplimit);
   fprintf(fid,"random: %d \n",ss_random);
   fprintf(fid,"musicfile: %s \n",ss_musicfile);
   fprintf(fid,"fullscreen: %d \n",ss_fullscreen);
   fprintf(fid,"replay: %d \n",ss_replay);

   fprintf(fid,"transitions data: \n");

   for (ii = 0; ii < SSNF; ii++)
      fprintf(fid,"%s %d %d %d \n", ss_trantab[ii].tranname,
              ss_trantab[ii].enabled, ss_trantab[ii].slowdown,
              ss_trantab[ii].preference);

   fprintf(fid,"images data: \n");

   for (ii = 0; ii < ss_Nfiles; ii++)
   {
      fprintf(fid,"imagefile: %s \n",ss_imagetab[ii].imagefile);
      fprintf(fid,"tone: %d \n",ss_imagetab[ii].tone);
      fprintf(fid,"caption: %d \n",ss_imagetab[ii].capsecs);
      fprintf(fid,"comments: %d \n",ss_imagetab[ii].commsecs);
      fprintf(fid,"wait1: %d \n",ss_imagetab[ii].wait1);
      fprintf(fid,"zoomsize: %.1f \n",ss_imagetab[ii].zoomsize);
      fprintf(fid,"zoomsteps: %d \n",ss_imagetab[ii].zoomsteps);
      fprintf(fid,"zoomloc: %d %d \n",ss_imagetab[ii].zoomlocx,ss_imagetab[ii].zoomlocy);
      fprintf(fid,"wait2: %d \n",ss_imagetab[ii].wait2);
      fprintf(fid,"tranname: %s \n",ss_imagetab[ii].tranname);
   }

   fclose(fid);
   return;
}


//  convert old preferences file format to new format
//  remove this code after Jan 2016   FIXME

void ss_conv_prefsfile()                                                   //  v.15.02
{
   FILE     *fid1, *fid2;
   char     buff[XFCC];
   char     old_prefsfile[300], new_prefsfile[300] = "";
   char     tranname[32], *pp;
   int      nn, cc, format = 0;
   int      n1, n2, n3, n4, n5, n6, n7;

   snprintf(old_prefsfile,300,"%s/%s",slideshow_dirk,ss_albumname);
   fid1 = fopen(old_prefsfile,"r");                                        //  open input slide show prefs file
   if (! fid1) return;

   strncatv(new_prefsfile,300,old_prefsfile,"-new",0);
   fid2 = fopen(new_prefsfile,"w");                                        //  open output converted file
   if (! fid2) {
      zmessageACK(Mwin,0,strerror(errno));
      fclose(fid1);
      return;
   }

   while (true)
   {
      pp = fgets_trim(buff,XFCC,fid1,1);
      if (! pp) break;

      if (strmatchN(pp,"overall:",8)) {
         fprintf(fid2,"global data: \n");
         format = 1;
         continue;
      }

      if (strmatchN(pp,"transitions:",12)) {
         fprintf(fid2,"transitions data: \n");
         format = 2;
         continue;
      }

      if (strmatchN(pp,"images:",7)) {
         fprintf(fid2,"images data: \n");
         format = 3;
         continue;
      }

      if (format == 1) {

         nn = sscanf(buff,"%d %d %d %d \"%m[^\"] ",&n1,&n2,&n3,&n4,&pp);
         if (nn != 5) goto format_error;
         fprintf(fid2,"seconds: %d \n",n1);
         fprintf(fid2,"cliplimit: %d \n",n2);
         fprintf(fid2,"random: %d \n",n4);
         fprintf(fid2,"musicfile: %s \n",pp);
         free(pp);
         continue;
      }

      if (format == 2) {
         nn = sscanf(buff,"%31s %d %d %d ",tranname,&n1,&n2,&n3);
         if (nn != 4) goto format_error;
         fprintf(fid2,"%s %d %d %d \n",tranname,n1,n2,n3);
         continue;
      }

      if (format == 3) {
         if (pp[0] != '"') goto format_error;                              //  rec. 1 = "/path.../filename"
         cc = strlen(pp);
         if (pp[cc-1] != '"') goto format_error;
         pp[cc-1] = 0;
         fprintf(fid2,"imagefile: %s \n",pp+1);
         pp = fgets_trim(buff,XFCC,fid1,1);                                //  rec. 2 = preferences
         if (! pp) goto format_error;
         nn = sscanf(buff,"%31s %d %d %d %d %d %d %d ",tranname,&n1,&n2,&n3,&n4,&n5,&n6,&n7);
         if (nn != 8) goto format_error;
         fprintf(fid2,"tone: %d \n",n1);
         fprintf(fid2,"wait1: %d \n",n2);
         fprintf(fid2,"zoomsize: %.1f \n",(n3+100)/100.0);                 //  conv 0-200 %  to  1-3 x
         fprintf(fid2,"zoomsteps: %d \n",n4);
         fprintf(fid2,"zoomloc: %d %d \n",n5,n6);
         fprintf(fid2,"wait2: %d \n",n7);
         fprintf(fid2,"tranname: %s \n",tranname);
         continue;
      }
   }

   fclose(fid1);
   fclose(fid2);

   rename(new_prefsfile,old_prefsfile);
   return;

format_error:
   zmessageACK(Mwin,0,ZTX("file format error: \n %s"),buff);
   fclose(fid1);
   fclose(fid2);
   return;
}


//  Show next slide when time is up or user navigates with arrow keys.
//  Cycles every 0.1 seconds when slide show is active.

int ss_timerfunc(void *)                                                   //  overhauled v.15.02
{
   int            ii, jj;
   int            capsecs, commsecs, mode;
   char           *file;
   double         cctime;
   static int     pmode;
   static double  starttime;

   if (ss_escape) {                                                        //  end slide show mode
      if (ss_pxbold) g_object_unref(ss_pxbold);                            //  free memory
      if (ss_pxbnew) g_object_unref(ss_pxbnew);
      ss_pxbold = ss_pxbnew = 0;
      if (*ss_musicfile == '/') shell_quiet("pulseaudio --kill");          //  kill music if any                  v.15.02
      win_unfullscreen();                                                  //  restore old window size, menu etc.
      ss_fullscreen = 0;
      Fslideshow = 0;                                                      //  reset flags
      Fblock = 0;
      Fblowup = 0;
      ss_escape = 0;
      ii = ss_Fcurrent;                                                    //  last file shown --> current file   v.15.04
      file = ss_imagetab[ii].imagefile;
      f_open(file);
      m_slideshow(0,0);                                                    //  return to slide show dialog
      return 0;                                                            //  stops the timer
   }

   if (ss_spacebar) {                                                      //  pause/resume
      ss_spacebar = 0;
      ss_blank = 0;
      if (! strmatch(ss_state,"pause"))                                    //  pause
         ss_state = "pause";
      else {                                                               //  resume
         if (zdmagnify) zdialog_send_event(zdmagnify,"kill");              //  if magnify active, terminate       v.14.12
         ss_Flast = ss_Fcurrent;
         ss_Fcurrent++;
         if (ss_Fcurrent == ss_Nfiles) ss_Fcurrent = 0;
         ss_state = "instant";
      }
      return 1;
   }

   if (ss_Mkey) {                                                          //  KB M key                           v.14.12
      ss_Mkey = 0;
      if (! strmatch(ss_state,"pause")) return 1;                          //  ignore if not paused
      ii = ss_Fcurrent;                                                    //  start magnify function
      f_open(ss_imagetab[ii].imagefile);
      m_magnify(0,0);
      return 1;
   }

   if (ss_Bkey) {                                                          //  blank/unblank screen
      ss_Bkey = 0;
      ss_blank = 1 - ss_blank;
      if (ss_blank) {
         ss_blankwindow();
         ss_state = "pause";
      }
      else ss_state = "instant";
      return 1;
   }

   if (ss_Larrow) {                                                        //  left arrow, back one image
      ss_Larrow = 0;
      ss_blank = 0;
      ss_Flast = ss_Fcurrent;
      ss_Fcurrent--;
      if (ss_Fcurrent < 0) ss_Fcurrent = ss_Nfiles-1;
      ss_state = "arrow";
      return 1;
   }

   if (ss_Rarrow) {                                                        //  right arrow, forward one image
      ss_Rarrow = 0;
      ss_blank = 0;
      ss_Flast = ss_Fcurrent;
      ss_Fcurrent++;
      if (ss_Fcurrent == ss_Nfiles) ss_Fcurrent = 0;
      ss_state = "arrow";
      return 1;
   }

   if (strmatch(ss_state,"pause")) return 1;                               //  do nothing

   if (strmatch(ss_state,"first"))                                         //  first image
      ss_state = "instant";

   if (strstr("show instant arrow",ss_state)) {                            //  show or instant show
      ii = ss_Fcurrent;                                                    //  current image
      ss_oldfile = ss_newfile;                                             //  old file = new
      if (ss_pxbold) g_object_unref(ss_pxbold);
      ss_pxbold = ss_pxbnew;                                               //  old pixbuf = new
      ss_rsold = ss_rsnew;
      ss_newfile = ss_imagetab[ii].imagefile;                              //  new current file
      ss_pxbnew = ss_loadpxb(ss_newfile);                                  //  new pixbuf
      if (! ss_pxbnew) return 0;
      ss_rsnew = gdk_pixbuf_get_rowstride(ss_pxbnew);

      if (strstr("instant arrow",ss_state))                                //  skip transition, show now
         ss_instant();
      else {
         jj = ss_nextrans();                                               //  select next transition type
         ss_slowdown = ss_trantab[jj].slowdown;                            //  set slowdown factor for transition
         ss_trantab[jj].func();                                            //  call transition function
      }

      if (! ss_fullscreen)                                                 //  set window title
         gtk_window_set_title(MWIN,ss_newfile);

      if (ss_imagetab[ii].tone)                                            //  play tone if specified             v.15.02
         shell_quiet("paplay %s/slideshow-tone.oga &",slideshow_dirk);

      if (strmatch(ss_state,"arrow")) {
         ss_state = "pause";
         return 1;
      }

      ss_state = "wait1";                                                  //  wait before zoom-in
      starttime = get_seconds();
      if (ss_imagetab[ii].capsecs > ss_imagetab[ii].commsecs)
         cctime = ss_imagetab[ii].capsecs;                                 //  add larger of caption/comments time
      else cctime = ss_imagetab[ii].commsecs;
      ss_timer = starttime + cctime + ss_imagetab[ii].wait1;               //  time to wait
      return 1;
   }

   if (strmatch(ss_state,"wait1")) {                                       //  show caption/comments
      cctime = get_seconds() - starttime;                                  //    for specified intervals          v.15.02
      ii = ss_Fcurrent;
      capsecs = ss_imagetab[ii].capsecs;
      commsecs = ss_imagetab[ii].commsecs;
      mode = 0;
      if (capsecs && capsecs > cctime) mode = 1;                           //  show caption
      if (commsecs && commsecs > cctime) mode += 2;                        //  show comments or both
      if ((capsecs || commsecs) && ! mode) mode = 4;                       //  erase them
      if (mode != pmode) ss_showcapcom(mode);
      pmode = mode;
   }

   if (strmatch(ss_state,"wait1")) {
      if (get_seconds() < ss_timer) return 1;                              //  wait for zoom-in time
      ii = ss_Fcurrent;
      ss_zoomsize = ss_imagetab[ii].zoomsize;                              //  1-3x = no zoom to 3x zoom
      ss_zoomsteps = ss_imagetab[ii].zoomsteps;                            //  zoom steps
      ss_zoomlocx = ss_imagetab[ii].zoomlocx;                              //  target location for final center
      ss_zoomlocy = ss_imagetab[ii].zoomlocy;                              //  (0-100% of image, 50/50 = middle)
      if (ss_zoomsize > 1.0) ss_zoomin();                                  //  do zoom if > 1x
      ss_state = "wait2";                                                  //  wait for next image
      ss_timer = get_seconds() + ss_imagetab[ii].wait2;
      return 1;
   }

   if (strmatch(ss_state,"wait2")) {
      if (get_seconds() < ss_timer) return 1;                              //  wait until next image time
      ss_state = "sswait";
      ss_timer = get_seconds() + ss_seconds;
      return 1;
   }

   if (strmatch(ss_state,"sswait")) {                                      //  global time interval
      if (get_seconds() < ss_timer) return 1;                              //  wait for my time
      ss_Flast = ss_Fcurrent;
      ss_Fcurrent++;
      if (ss_Fcurrent == ss_Nfiles) ss_Fcurrent = 0;                       //  reached end, back to first image
      if (ss_Fcurrent == 0 && ! ss_replay) ss_escape = 1;                  //  exit if no auto-replay
      ss_state = "show";
      return 1;
   }

   return 1;
}


//  select next transition type to use
//  mode = sequential: use each enabled transition type in sequence
//  mode = random: exclude recently used, choose random from remaining

int ss_nextrans()
{
   int      ii, jj, maxii, maxjj, next;
   float    maxrank, rank;

   ii = ss_Flast;                                                          //  transition type from last image

   if (! strmatch(ss_imagetab[ii].tranname,"next")) {                      //  image transition not "next"
      for (jj = 0; jj < SSNF; jj++)
         if (strmatch(ss_trantab[jj].tranname,ss_imagetab[ii].tranname)) break;
      if (jj < SSNF) {
         next = jj;                                                        //  assigned transition type
         for (ii = ss_Nused - 1; ii > 0; ii--)                             //  >> most recently used
            ss_Tlast[ii] = ss_Tlast[ii-1];
         ss_Tlast[0] = next;
         return next;
      }
   }

   if (ss_Nused < 5 || ss_random == 0)                                     //  few enabled transition types
   {                                                                       //    or sequential mode
      ss_Tnext++;
      if (ss_Tnext == ss_Nused) ss_Tnext = 0;                              //  select transition types sequentially
      next = ss_Tused[ss_Tnext];
   }

   else                                                                    //  select transition types randomly
   {
      maxrank = 0;
      maxii = 0;
      maxjj = ss_Nused / 2;                                                //  most recently used to exclude
      if (maxjj > 4) maxjj = 4;                                            //  max. 4

      for (ii = 0; ii < ss_Nused; ii++)                                    //  search enabled transitions
      {
         for (jj = 0; jj < maxjj; jj++)                                    //  exclude most recently used 50%
            if (ss_Tused[ii] == ss_Tlast[jj]) break;
         if (jj < maxjj) continue;
         jj = ss_Tused[ii];
         rank = ss_trantab[jj].preference * drandz();                      //  rank = preference * random value
         if (rank > maxrank) {
            maxrank = rank;                                                //  remember highest rank
            maxii = ii;
         }
      }

      next = ss_Tused[maxii];                                              //  transition to use

      for (ii = ss_Nused - 1; ii > 0; ii--)                                //  make it most recent
         ss_Tlast[ii] = ss_Tlast[ii-1];
      ss_Tlast[0] = next;
   }

   return next;
}


//  write captions and comments at the top of the image
//  mode: 1 write caption only
//        2 write comments only
//        3 write both
//        4 clear both

void ss_showcapcom(int mode)                                               //  v.15.02
{
   cchar        *keynames[2] = { iptc_caption_key, exif_comment_key };
   char         *keyvals[2];
   char         caption[200], comments[200];
   static char  text[402];
   PIXBUF       *pxbclear;

   static PangoFontDescription   *pangofont = null;
   static PangoLayout            *pangolayout = null;
   static int                    plww, plhh;

   if (plww) {                                                             //  clear previous text
      ss_cr = gdk_cairo_create(gdkwin);                                    //  (restore image)
      pxbclear = gdk_pixbuf_new_subpixbuf(ss_pxbnew,0,0,plww+10,plhh+10);
      gdk_cairo_set_source_pixbuf(ss_cr,pxbclear,0,0);
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
      g_object_unref(pxbclear);
      plww = 0;
   }

   if (mode == 4) return;

   *caption = *comments = 0;
   exif_get(ss_newfile,keynames,keyvals,2);                                //  get captions and comments metadata

   if (keyvals[0]) {
      strncpy0(caption,keyvals[0],200);
      zfree(keyvals[0]);
   }

   if (keyvals[1]) {
      strncpy0(comments,keyvals[1],200);
      zfree(keyvals[1]);
   }

   *text = 0;

   if (mode == 1 && *caption) strcpy(text,caption);                        //  show caption only
   if (mode == 2 && *comments) strcpy(text,comments);                      //  show comments only
   if (mode == 3) {
      if (*caption) strcpy(text,caption);                                  //  show both on two lines
      if (*comments && *caption) strcat(text,"\n");
      if (*comments) strcat(text,comments);
   }

   if (! *text) return;

   for (int ii = 0; text[ii]; ii++)                                        //  replace "\n" with real newlines    v.15.01.1
      if (text[ii] == '\\' && text[ii+1] == 'n')
         memmove(text+ii,"\n ",2);

   pangofont = pango_font_description_from_string("Sans 12");              //  make pango layout for font
   pangolayout = gtk_widget_create_pango_layout(Cdrawin,0);
   pango_layout_set_font_description(pangolayout,pangofont);
   pango_layout_set_text(pangolayout,text,-1);                             //  add text to layout
   pango_layout_get_pixel_size(pangolayout,&plww,&plhh);

   ss_cr = gdk_cairo_create(gdkwin);

   cairo_set_line_width(ss_cr,1);
   cairo_set_source_rgb(ss_cr,1,1,1);                                      //  draw white background
   cairo_rectangle(ss_cr,10,10,plww,plhh);
   cairo_fill(ss_cr);

   cairo_move_to(ss_cr,10,10);                                             //  draw layout with text
   cairo_set_source_rgb(ss_cr,0,0,0);
   pango_cairo_show_layout(ss_cr,pangolayout);

   cairo_destroy(ss_cr);
   return;
}


//  Load image and rescale to fit in window size.
//  If image aspect ratio is close enough to window ratio,
//  truncate to avoid having margins around around the image.

PIXBUF * ss_loadpxb(char *file)                                            //  v.14.02
{
   PIXBUF      *pxbin, *pxbtemp, *pxbout;
   int         ww1, hh1, ww2, hh2;
   int         Iorgx, Iorgy, Worgx, Worgy;
   float       Rm, Rw, dR;
   GError      *gerror = 0;

   if (ss_fullscreen) {
      ss_ww = screenww;                                                    //  drawing window size
      ss_hh = screenhh;                                                    //  v.14.08
   }
   else {
      ss_ww = Dww;                                                         //  drawing window size
      ss_hh = Dhh;
   }

   pxbin = gdk_pixbuf_new_from_file(file,&gerror);                         //  load image file into pixbuf
   if (! pxbin) {
      zmessageACK(Mwin,0,"%s",gerror->message);
      return 0;
   }

   ww1 = gdk_pixbuf_get_width(pxbin);                                      //  image dimensions
   hh1 = gdk_pixbuf_get_height(pxbin);

   ww2 = ss_ww;                                                            //  window dimensions
   hh2 = ss_hh;

   Rm = 1.0 * ww1 / hh1;                                                   //  image width/height ratio
   Rw = 1.0 * ww2 / hh2;                                                   //  window width/height ratio
   dR = fabsf(Rm - Rw) / Rw;                                               //  discrepancy ratio

   if (dR <= 0.01 * ss_cliplimit) {                                        //  discrepancy within user limit
      if (Rw >= Rm) {
         ww1 = ww2;                                                        //  height will be clipped
         hh1 = ww1 / Rm;
      }
      else {
         hh1 = hh2;                                                        //  width will be clipped
         ww1 = hh1 * Rm;
      }
   }
   else {                                                                  //  discrepancy too great
      if (Rw >= Rm) {
         hh1 = hh2;                                                        //  ratio image to fit in window
         ww1 = hh1 * Rm;
      }
      else {
         ww1 = ww2;
         hh1 = ww1 / Rm;
      }
   }

   pxbtemp = gdk_pixbuf_scale_simple(pxbin,ww1,hh1,BILINEAR);              //  rescale image
   g_object_unref(pxbin);

   Iorgx = (ww1 - ww2) / 2.0;                                              //  top left corner of image to copy from
   if (Iorgx < 0) Iorgx = 0;
   Iorgy = (hh1 - hh2) / 2.0;
   if (Iorgy < 0) Iorgy = 0;

   Worgx = (ww2 - ww1) / 2.0;                                              //  top left corner of window to copy to
   if (Worgx < 0) Worgx = 0;
   Worgy = (hh2 - hh1) / 2.0;
   if (Worgy < 0) Worgy = 0;

   if (ww2 < ww1) ww1 = ww2;                                               //  copy width
   if (hh2 < hh1) hh1 = hh2;                                               //  copy height

   pxbout = gdk_pixbuf_new(GDKRGB,0,8,ww2,hh2);                            //  output pixbuf = window size        v.15.01
   gdk_pixbuf_fill(pxbout,0);                                              //  black margins
   gdk_pixbuf_copy_area(pxbtemp,Iorgx,Iorgy,ww1,hh1,pxbout,Worgx,Worgy);   //  insert image
   g_object_unref(pxbtemp);

   return pxbout;
}


//  write black to entire window

void ss_blankwindow()
{
   GdkRGBA     GDKdark;

   GDKdark.red = GDKdark.green = GDKdark.blue = 0.2;
   GDKdark.alpha = 1;
   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_rgba(ss_cr,&GDKdark);
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);
   gtk_main_iteration_do(0);
   return;
}


//  instant transition (also used for keyboard arrow keys)

void ss_instant()
{
   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);
   gtk_main_iteration_do(0);
   return;
}


//  fade-out / fade-in transition

void ss_fadein()
{
   PIXBUF      *pxbmix;
   int         ii, jj, kk, px, py, rs, iinc;
   float       newpart, oldpart;
   uint8       *pixels1, *pixels2, *pixels3;
   uint8       *pix1, *pix2, *pix3;

   pxbmix = gdk_pixbuf_copy(ss_pxbold);
   rs = gdk_pixbuf_get_rowstride(pxbmix);

   pixels1 = gdk_pixbuf_get_pixels(ss_pxbold);
   pixels2 = gdk_pixbuf_get_pixels(ss_pxbnew);
   pixels3 = gdk_pixbuf_get_pixels(pxbmix);

   iinc = 10.0 / (1 + ss_slowdown / 2.0);                                  //  slowdown factor                    v.14.01
   if (iinc < 1) iinc = 1;

   for (ii = 0; ii <= 100; ii += iinc)
   {
      newpart = 0.01 * ii;
      oldpart = 1.0 - newpart;

      for (jj = 0; jj < 2; jj++)                                           //  four passes, each modifies 25%
      for (kk = 0; kk < 2; kk++)                                           //    of the pixels (visually smoother)
      {
         for (py = jj; py < ss_hh; py += 2)
         for (px = kk; px < ss_ww; px += 2)
         {
            pix1 = pixels1 + py * ss_rsold + px * 3;
            pix2 = pixels2 + py * ss_rsnew + px * 3;
            pix3 = pixels3 + py * rs + px * 3;
            pix3[0] = newpart * pix2[0] + oldpart * pix1[0];
            pix3[1] = newpart * pix2[1] + oldpart * pix1[1];
            pix3[2] = newpart * pix2[2] + oldpart * pix1[2];
         }

         ss_cr = gdk_cairo_create(gdkwin);
         gdk_cairo_set_source_pixbuf(ss_cr,pxbmix,0,0);
         cairo_paint(ss_cr);
         cairo_destroy(ss_cr);
         gtk_main_iteration_do(0);
      }
   }

   g_object_unref(pxbmix);
   return;
}


//  new image rolls over prior image from left to right

void ss_rollright()
{
   PIXBUF      *pixbuf;
   int         px;
   float       delay = 1.0 * ss_slowdown / ss_ww;                          //  v.14.01
   uint8       *pixels, *pix3;

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   for (px = 0; px < ss_ww-4; px += 4)                                     //  4-wide
   {
      pix3 = pixels + px * 3;
      pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,4,ss_hh,ss_rsnew,0,0);
      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,0);
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      g_object_unref(pixbuf);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   return;
}


//  new image rolls over prior image from top down

void ss_rolldown()
{
   PIXBUF      *pixbuf;
   int         py;
   float       delay = 1.0 * ss_slowdown / ss_hh;                          //  v.14.01
   uint8       *pixels, *pix3;

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   for (py = 0; py < ss_hh-2; py += 4)                                     //  4-deep
   {
      pix3 = pixels + py * ss_rsnew;
      pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ss_ww,4,ss_rsnew,0,0);
      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,0,py);
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      g_object_unref(pixbuf);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   return;
}


//  new image opens up in horizontal rows like venetian blinds

void ss_venetian()
{
   PIXBUF      *pixbuf;
   int         py1, py2;
   uint8       *pixels, *pix3;
   int         louver, Nlouvers = 20;
   int         louversize = ss_hh / Nlouvers;
   float       delay = 1.0 / louversize * (1 + ss_slowdown / 2.0);         //  v.14.01

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   for (py1 = 0; py1 < louversize; py1++)                                  //  y-row within each louver
   {
      ss_cr = gdk_cairo_create(gdkwin);

      for (louver = 0; louver < Nlouvers; louver++)                        //  louver, first to last
      {
         py2 = py1 + louver * louversize;
         if (py2 >= ss_hh) break;
         pix3 = pixels + py2 * ss_rsnew;
         pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ss_ww,1,ss_rsnew,0,0);
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,0,py2);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);
      }

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   return;
}


//  a grate opens up to show new image

void ss_grate()
{
   PIXBUF      *pixbuf;
   int         px1, px2, py1, py2;
   uint8       *pixels, *pix3;
   int         row, col, Nrow, Ncol;                                       //  rows and columns
   int         boxww, boxhh;
   float       delay;

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   Ncol = 20;                                                              //  20 columns
   boxww = boxhh = ss_ww / Ncol;                                           //  square boxes
   Nrow = ss_hh / boxhh;                                                   //  corresp. rows
   Ncol++;                                                                 //  round up
   Nrow++;
   delay = 1.0 / boxhh * (1 + ss_slowdown / 2.0);                          //  v.14.01

   for (py1 = 0; py1 < boxhh; py1++)
   {
      ss_cr = gdk_cairo_create(gdkwin);

      for (row = 0; row < Nrow; row++)
      {
         py2 = py1 + row * boxhh;
         if (py2 >= ss_hh) break;
         pix3 = pixels + py2 * ss_rsnew;
         pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ss_ww,1,ss_rsnew,0,0);
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,0,py2);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);
      }

      px1 = py1;

      for (col = 0; col < Ncol; col++)
      {
         px2 = px1 + col * boxww;
         if (px2 >= ss_ww) break;
         pix3 = pixels + px2 * 3;
         pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,1,ss_hh,ss_rsnew,0,0);
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px2,0);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);
      }

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   return;
}


//  A rectangle opens up from the center and expands outward

void ss_rectangle()
{
   PIXBUF      *pixbuf;
   int         px1, py1, px2, py2, px3, py3;
   int         ww1, hh1, ww2, hh2;
   uint8       *pixels, *pix3;
   int         step, Nsteps = 200;
   float       delay = 1.0 / Nsteps * (1 + ss_slowdown / 2.0);             //  v.14.01

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   for (step = 1; step < Nsteps; step++)
   {
      ww1 = ss_ww * step / Nsteps;
      hh1 = ww1 * ss_hh / ss_ww;
      ww2 = ss_ww / Nsteps / 2 + 1;
      hh2 = ss_hh / Nsteps / 2 + 1;

      px1 = (ss_ww - ww1) / 2;
      py1 = (ss_hh - hh1) / 2;
      px2 = px1 + ww1 - ww2;
      py2 = py1;
      px3 = px1;
      py3 = py1 + hh1 - hh2;

      ss_cr = gdk_cairo_create(gdkwin);

      pix3 = pixels + py1 * ss_rsnew + px1 * 3;
      pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww1+1,hh2+1,ss_rsnew,0,0);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px1,py1);
      cairo_paint(ss_cr);
      g_object_unref(pixbuf);

      pix3 = pixels + py2 * ss_rsnew + px2 * 3;
      pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww2+1,hh1+1,ss_rsnew,0,0);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px2,py2);
      cairo_paint(ss_cr);
      g_object_unref(pixbuf);

      pix3 = pixels + py3 * ss_rsnew + px3 * 3;
      pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww1+1,hh2+1,ss_rsnew,0,0);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px3,py3);
      cairo_paint(ss_cr);
      g_object_unref(pixbuf);

      pix3 = pixels + py1 * ss_rsnew + px1 * 3;
      pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww2+1,hh1+1,ss_rsnew,0,0);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px1,py1);
      cairo_paint(ss_cr);
      g_object_unref(pixbuf);

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);                       //  v.14.03
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);

   return;
}


//  old image shrinks to the center, revealing new image

void ss_implode()                                                          //  v.15.03
{
   int         Nsteps = 100 + 30 * ss_slowdown;
   int         ww, hh, px, py;
   float       size = 1.0, F;
   PIXBUF      *pxbnew, *pxbold;

   F = 1.0 * (Nsteps-1) / Nsteps;

   while (true)
   {
      pxbnew = gdk_pixbuf_copy(ss_pxbnew);                                 //  new image at full size

      size = F * pow(size,1.02);
      if (size < 0.02) break;

      ww = ss_ww * size;
      hh = ss_hh * size;
      pxbold = pixbuf_rescale_fast(ss_pxbold,ww,hh);                       //  old image at reduced size

      px = ss_ww * 0.5 * (1.0 - size);                                     //  new image position, NW corner --> center
      py = ss_hh * 0.5 * (1.0 - size);
      gdk_pixbuf_copy_area(pxbold,0,0,ww,hh,pxbnew,px,py);                 //  copy shrinking old image into new image

      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pxbnew,0,0);                       //  paint new image
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);

      g_object_unref(pxbnew);
      g_object_unref(pxbold);
   }

   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);

   return;
}


//  new image grows from the center, covering old image

void ss_explode()                                                          //  v.15.04
{
   int         Nsteps = 60 + 20 * ss_slowdown;
   int         ww, hh, px, py;
   float       size = 0.05, F;
   PIXBUF      *pxbnew, *pxbold;

   F = 1.0 * (Nsteps+1) / Nsteps;

   while (true)
   {
      pxbold = gdk_pixbuf_copy(ss_pxbold);                                 //  old image at full size

      size = F * pow(size,0.995);
      if (size > 1.0) break;

      ww = ss_ww * size;
      hh = ss_hh * size;
      pxbnew = pixbuf_rescale_fast(ss_pxbnew,ww,hh);                       //  new image at reduced size

      px = ss_ww * 0.5 * (1.0 - size);                                     //  new image position, center --> NW corner
      py = ss_hh * 0.5 * (1.0 - size);
      gdk_pixbuf_copy_area(pxbnew,0,0,ww,hh,pxbold,px,py);                 //  copy shrinking old image into new image

      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pxbold,0,0);                       //  paint new image
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);

      g_object_unref(pxbnew);
      g_object_unref(pxbold);
   }

   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);

   return;
}


//  New image sweeps into view like a circular radar image

void ss_radar()                                                            //  v.15.04
{
   int         px, py, ww, hh;
   int         px1, py1, px2, py2;
   int         ww2 = ss_ww/2, hh2 = ss_hh/2;
   float       R, Rmax, dR, T, dT;
   float       r, r1, r2;
   float       cosT, sinT;
   uint8       *pixels1, *pixels3, *pix1, *pix3;
   GdkPixbuf   *pixbuf;

   px = py = 0;                                                            //  suppress compiler warnings

   if (! resource_lock(Fpaintlock)) return;
   gdk_window_freeze_updates(gdkwin);

   pixels1 = gdk_pixbuf_get_pixels(ss_pxbold);
   pixels3 = gdk_pixbuf_get_pixels(ss_pxbnew);

   Rmax = sqrt(ww2*ww2 + hh2*hh2);                                         //  max. line length, center to corner
   dR = 100;                                                               //  line segment length
   dT = 1.0 / Rmax;                                                        //  angle step

   for (T = 0; T < 2*PI; T += dT)                                          //  angle from 0 to 360 deg.
   {
      cosT = cosf(T);
      sinT = sinf(T);

      ss_cr = gdk_cairo_create(gdkwin);

      for (R = 0; R < Rmax; R += dR)                                       //  R from center to edge
      {
         r1 = R;                                                           //  R segment
         r2 = R + dR;

         px1 = ww2 + r1 * cosT;                                            //  R segment
         px2 = ww2 + r2 * cosT;
         py1 = hh2 - r1 * sinT;
         py2 = hh2 - r2 * sinT;

         for (r = r1; r <= r2; r++)                                        //  loop R segment pixels
         {
            px = ww2 + r * cosT;
            py = hh2 - r * sinT;

            if (px < 0) px = 0;
            if (px > ss_ww-2) px = ss_ww-2;
            if (py < 0) py = 0;
            if (py > ss_hh-1) py = ss_hh-1;

            pix1 = pixels1 + py * ss_rsold + px * 3;                       //  copy new image pixel to old image
            pix3 = pixels3 + py * ss_rsnew + px * 3;                       //  right side
            memcpy(pix1,pix3,6);

            if (px == 0 || px == ss_ww-2) break;                           //  reached edge of image
            if (py == 0 || py == ss_hh-1) break;
         }

         px2 = px;                                                         //  actual end of R segment
         py2 = py;

         if (px1 < px2) px = px1;                                          //  get rectangle enclosing R segment
         else px = px2;
         if (py1 < py2) py = py1;
         else py = py2;
         ww = abs(px2-px1) + 1;
         hh = abs(py2-py1) + 1;
         if (ww <= 0) break;
         if (hh <= 0) break;

         if (px < 0) px = 0;
         if (px > ss_ww-1) px = ss_ww-1;
         if (px + ww > ss_ww) ww = ss_ww - px;

         if (py < 0) py = 0;
         if (py > ss_hh-1) py = ss_hh-1;
         if (py + hh > ss_hh) hh = ss_hh - py;

         pixbuf = gdk_pixbuf_new_subpixbuf(ss_pxbold,px,py,ww,hh);         //  paint window rectangle
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,py);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);
      }

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);

      zsleep(0.0001 * ss_slowdown);
   }

   resource_unlock(Fpaintlock);
   gdk_window_thaw_updates(gdkwin);
   return;
}


//  new image opens up like a Japanese fan

void ss_japfan()                                                           //  v.15.04
{
   int         px, py, pxL, ww, hh;
   int         px1, py1, px2, py2;
   int         ww2 = ss_ww/2, hh2 = ss_hh/2;
   float       R, Rmax, dR, T, dT;
   float       r, r1, r2;
   float       cosT, sinT;
   uint8       *pixels1, *pixels3, *pix1, *pix3;
   GdkPixbuf   *pixbuf;

   px = py = 0;                                                            //  suppress compiler warnings

   if (! resource_lock(Fpaintlock)) return;
   gdk_window_freeze_updates(gdkwin);

   pixels1 = gdk_pixbuf_get_pixels(ss_pxbold);
   pixels3 = gdk_pixbuf_get_pixels(ss_pxbnew);

   Rmax = sqrt(ww2*ww2 + hh2*hh2);                                         //  max. line length, center to corner
   dR = 100;                                                               //  line segment length
   dT = 1.0 / Rmax;                                                        //  angle step

   for (T = PI/2; T > -PI/2; T -= dT)                                      //  angle from +90 to -90 deg.
   {
      cosT = cosf(T);
      sinT = sinf(T);

      ss_cr = gdk_cairo_create(gdkwin);

      for (R = 0; R < Rmax; R += dR)                                       //  R from center to edge
      {
         r1 = R;                                                           //  R segment
         r2 = R + dR;

         px1 = ww2 + r1 * cosT;                                            //  R segment
         px2 = ww2 + r2 * cosT;
         py1 = hh2 - r1 * sinT;
         py2 = hh2 - r2 * sinT;

         for (r = r1; r <= r2; r++)                                        //  loop R segment pixels
         {
            px = ww2 + r * cosT;
            py = hh2 - r * sinT;

            if (px > ss_ww-2) px = ss_ww-2;
            if (py < 0) py = 0;
            if (py > ss_hh-1) py = ss_hh-1;

            pix1 = pixels1 + py * ss_rsold + px * 3;                       //  copy new image pixel to old image
            pix3 = pixels3 + py * ss_rsnew + px * 3;                       //  right side
            memcpy(pix1,pix3,6);

            pxL = ss_ww-1 - px;

            pix1 = pixels1 + py * ss_rsold + pxL * 3;                      //  left side pixel
            pix3 = pixels3 + py * ss_rsnew + pxL * 3;
            memcpy(pix1,pix3,6);

            if (pxL == 0) break;                                           //  reached edge of image
            if (py == 0 || py == ss_hh-1) break;
         }

         px2 = px;                                                         //  actual end of R segment
         py2 = py;

         px = px1;
         if (py1 < py2) py = py1;                                          //  get rectangle enclosing R segment
         else py = py2;
         ww = px2 - px1 + 1;
         hh = abs(py2-py1) + 1;
         if (ww <= 0) break;
         if (hh <= 0) break;

         if (px < 0) px = 0;
         if (px > ss_ww-1) px = ss_ww-1;
         if (px + ww > ss_ww) ww = ss_ww - px;

         if (py < 0) py = 0;
         if (py > ss_hh-1) py = ss_hh-1;
         if (py + hh > ss_hh) hh = ss_hh - py;

         pixbuf = gdk_pixbuf_new_subpixbuf(ss_pxbold,px,py,ww,hh);         //  paint window rectangle
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,py);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);

         pxL = ss_ww-1 - px2;

         pixbuf = gdk_pixbuf_new_subpixbuf(ss_pxbold,pxL,py,ww,hh);        //  left side rectangle
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,pxL,py);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);
      }

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);

      zsleep(0.0001 * ss_slowdown);
   }

   resource_unlock(Fpaintlock);
   gdk_window_thaw_updates(gdkwin);
   return;
}


//  New image closes in from top and bottom with jagged teeth

void ss_jaws()
{
   PIXBUF      *pixbuf;
   int         nteeth = 20, Np = 10;
   int         tbase1, tbase2, twidth, tlength, tooth, tpos;
   int         ii, px, py, ww, ww2;
   float       delay = 0.0005 * ss_slowdown;
   uint8       *pixels, *pix3;

   ss_cr = gdk_cairo_create(gdkwin);
   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   twidth = ss_ww / nteeth;
   tlength = twidth;

   Np = Np / (1.0 + ss_slowdown / 4.0);                                    //  v.14.01
   if (Np < 1) Np = 1;

   for (ii = 0; ii <= ss_hh/2 - tlength/2 + 1; ii += Np)
   {
      tbase1 = ii;                                                         //  tooth base from window top to middle
      tbase2 = ss_hh - tbase1 - 1;                                         //  tooth base from window bottom to middle

      for (tooth = 0; tooth <= nteeth; tooth++)                            //  tooth first to last + 1
      {
         for (tpos = 0; tpos < tlength; tpos += Np)                        //  tooth position from base to point
         {
            ww = twidth * (tlength - tpos) / tlength;                      //  tooth width at scan line
            if (ww < 2) break;

            py = tbase1 + tpos;                                            //  top teeth scan line y
            px = twidth / 2 + tooth * twidth - ww / 2;                     //  scan line x to x + ww
            if (px < ss_ww) {
               ww2 = ww;
               if (px + ww2 > ss_ww) ww2 = ss_ww - px;
               pix3 = pixels + py * ss_rsnew + px * 3;
               pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww2,Np,ss_rsnew,0,0);
               gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,py);
               cairo_paint(ss_cr);
               g_object_unref(pixbuf);

            }

            py = tbase2 - tpos;                                            //  bottom teeth scan line y
            py = py - Np;
            px = tooth * twidth - ww / 2;                                  //  scan line x to x + ww
            if (tooth == 0) {
               px = 0;                                                     //  leftmost tooth is half
               ww = ww / 2;
            }
            if (px < ss_ww) {
               ww2 = ww;
               if (px + ww2 > ss_ww) ww2 = ss_ww - px;
               pix3 = pixels + py * ss_rsnew + px * 3;
               pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww2,Np,ss_rsnew,0,0);
               gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,py);
               cairo_paint(ss_cr);
               g_object_unref(pixbuf);

            }
         }
      }
      zsleep(delay);                                                       //  v.15.04
   }

   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);                       //  v.14.03
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);
   return;
}


//  An ellipse opens up from the center and expands outward

void ss_ellipse()
{
   PIXBUF      *pixbuf;
   uint8       *pixels, *pix3;
   int         step, Nsteps = 100;
   int         px1, py1, ww;
   float       delay = 0.1 / Nsteps;
   float       a, b, a2, b2, px, py, px2, py2;
   float       ww2 = ss_ww / 2, hh2 = ss_hh / 2;

   delay = delay * (1 + ss_slowdown / 2.0);

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   for (step = 1; step < 1.4 * Nsteps; step++)
   {
      a = ww2 * step / Nsteps;                                             //  ellipse a and b constants
      b = a * ss_hh / ss_ww;                                               //    from tiny to >> image size
      a2 = a * a;
      b2 = b * b;

      ss_cr = gdk_cairo_create(gdkwin);

      for (py = -b; py <= +b; py += 3)                                     //  py from top of ellipse to bottom
      {
         while (py < -(hh2-2)) py += 3;
         if (py > hh2-2) break;
         py2 = py * py;
         px2 = a2 * (1.0 - py2 / b2);                                      //  corresponding px value,
         px = sqrt(px2);                                                   //  (+/- from center of ellipse)
         if (px > ww2) px = ww2;
         ww = 2 * px;                                                      //  length of line thru ellipse
         if (ww < 2) continue;
         px1 = ww2 - px;                                                   //  relocate origin
         py1 = py + hh2;
         if (px1 + ww > ss_ww) px1 = ss_ww - ww;                           //  insurance
         if (py1 + 3 > ss_hh) py1 = ss_hh - 3;
         pix3 = pixels + py1 * ss_rsnew + px1 * 3;
         pixbuf = gdk_pixbuf_new_from_data(pix3,GDKRGB,0,8,ww,3,ss_rsnew,0,0);
         gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px1,py1);
         cairo_paint(ss_cr);
         g_object_unref(pixbuf);
      }

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);                       //  v.14.03
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);

   return;
}


//  new image splats onto old image one drop at a time

void ss_raindrops()
{
   PIXBUF      *pxbmix, *pxbdrop;
   int         rsmix;
   int         px, py, px1, py1, px2, py2, cx, cy;
   int         Rmin, Rmax, R, R2, dist2, Ndrops;
   uint8       *pixels2, *pixels3;
   uint8       *pix2, *pix3 = 0;
   float       dtime;

   pixels2 = gdk_pixbuf_get_pixels(ss_pxbnew);                             //  source image

   pxbmix = gdk_pixbuf_new(GDKRGB,0,8,ss_ww,ss_hh);                        //  destination image
   pixels3 = gdk_pixbuf_get_pixels(pxbmix);
   rsmix = gdk_pixbuf_get_rowstride(pxbmix);
   memset(pixels3,0,ss_hh * rsmix);                                        //  clear dest. to black

   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,pxbmix,0,0);                          //  paint black window
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);

   Rmin = ss_ww * 0.01;                                                    //  drop size range
   Rmax = ss_ww * 0.02;
   Ndrops = 3000;

   for (int ii = 0; ii < Ndrops; ii++)
   {
      cx = drandz() * ss_ww;                                               //  drop location on image
      cy = drandz() * ss_hh;
      R = drandz() * Rmax + Rmin;                                          //  drop size
      R2 = R * R;
      px1 = cx - R;
      if (px1 < 0) px1 = 0;
      py1 = cy - R;
      if (py1 < 0) py1 = 0;
      px2 = cx + R;
      if (px2 >= ss_ww) px2 = ss_ww;
      py2 = cy + R;
      if (py2 > ss_hh) py2 = ss_hh;

      for (py = py1; py < py2; py++)                                       //  copy drop area from new image
      for (px = px1; px < px2; px++)                                       //    to old image
      {
         dist2 = (px-cx) * (px-cx) + (py-cy) * (py-cy);
         if (dist2 > R2) continue;
         pix2 = pixels2 + py * ss_rsnew + px * 3;
         pix3 = pixels3 + py * rsmix + px * 3;
         memcpy(pix3,pix2,3);                                              //  v.14.11
      }

      pxbdrop = gdk_pixbuf_new_subpixbuf(pxbmix,px1,py1,px2-px1,py2-py1);
      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pxbdrop,px1,py1);
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);

      g_object_unref(pxbdrop);
      dtime = 0.001 * (1.0 - pow(1.0*ii/Ndrops,0.1));
      dtime = dtime * (1.0 + ss_slowdown);                                 //  v.14.01
      zsleep(dtime);
      gtk_main_iteration_do(0);
   }

   ss_cr = gdk_cairo_create(gdkwin);
   gdk_cairo_set_source_pixbuf(ss_cr,ss_pxbnew,0,0);                       //  fill bits that are still missing   v.14.02
   cairo_paint(ss_cr);
   cairo_destroy(ss_cr);
   g_object_unref(pxbmix);
   return;
}


//  new image spreads from the middle to left and right edges
//  like a double-door swinging open

void ss_doubledoor()
{
   #define GPNFD(pix,ww,hh) gdk_pixbuf_new_from_data(pix,GDKRGB,0,8,ww,hh,ss_rsnew,0,0)

   PIXBUF      *pixbuf;
   int         bx, px;
   uint8       *pixels, *pix3;
   float       delay = 0.2 / ss_ww;

   delay = delay * (1 + ss_slowdown / 2.0);                                //  v.14.01

   pixels = gdk_pixbuf_get_pixels(ss_pxbnew);

   for (bx = 0; bx < ss_ww/2; bx++)                                        //  bx = 0 ... ww/2
   {
      ss_cr = gdk_cairo_create(gdkwin);

      px = ss_ww / 2 - bx;
      pix3 = pixels + 3 * px;                                              //  line from (ww/2-bx,0) to (ww/2-bx,hh-1)
      pixbuf = GPNFD(pix3,1,ss_hh);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,0);
      cairo_paint(ss_cr);
      g_object_unref(pixbuf);

      px = ss_ww / 2 + bx;
      pix3 = pixels + 3 * px;                                              //  line from (ww/2+bx,0) to (ww/2+bx,hh-1)
      pixbuf = GPNFD(pix3,1,ss_hh);
      gdk_cairo_set_source_pixbuf(ss_cr,pixbuf,px,0);
      cairo_paint(ss_cr);
      g_object_unref(pixbuf);

      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
      zsleep(delay);
   }

   return;
}


//  Rotate from old image to new image.

namespace ss_rotate_names
{
   int      cx1, cy1, cx2, cy2, cy3, cy4;
   uint8    *pixels1, *pixels2, *pixels3;
   int      rsmix, tbusy[max_threads];
}

void ss_rotate()                                                           //  v.14.02
{
   using namespace ss_rotate_names;

   void * ss_rotate_thread1(void *arg);
   void * ss_rotate_thread2(void *arg);

   PIXBUF      *pxbmix;
   float       R, step, stepx = 1, Nsteps = 40;
   int         ii;

   pixels1 = gdk_pixbuf_get_pixels(ss_pxbold);                             //  source images
   pixels2 = gdk_pixbuf_get_pixels(ss_pxbnew);

   pxbmix = gdk_pixbuf_new(GDKRGB,0,8,ss_ww,ss_hh);                        //  destination image
   rsmix = gdk_pixbuf_get_rowstride(pxbmix);
   pixels3 = gdk_pixbuf_get_pixels(pxbmix);

   Nsteps = Nsteps * (1.0 + ss_slowdown / 2.0);

   if (! resource_lock(Fpaintlock)) return;                                //  v.15.02
   gdk_window_freeze_updates(gdkwin);

   for (step = 0; step < Nsteps; step += stepx)
   {
      R = 1.0 * step / Nsteps;
      stepx = 2 - R;                                                       //  2 ... 1

      if (step + stepx >= Nsteps) {
         step = Nsteps;
         R = 1.0;
      }

      cx1 = R * ss_ww / 2.0;                                               //  corners of shrinking trapezoid
      cy1 = 0.3 * R * ss_hh;
      cx2 = ss_ww - cx1;
      cy2 = 0;
      cy3 = ss_hh;
      cy4 = ss_hh - cy1;

      memset(pixels3,0,ss_hh * rsmix);

      for (ii = 0; ii < NWT; ii++) {                                       //  start worker threads
         tbusy[ii] = 1;
         start_detached_thread(ss_rotate_thread1,&Nval[ii]);
      }

      for (ii = 0; ii < NWT; ii++)
         while(tbusy[ii]) zsleep(0.001);

      ss_cr = gdk_cairo_create(gdkwin);                                    //  create and destroy each paint      v.15.02
      gdk_cairo_set_source_pixbuf(ss_cr,pxbmix,0,0);
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
   }

   for (step = 0; step < Nsteps; step += stepx)
   {
      R = 1.0 * step / Nsteps;
      stepx = 1 + R;                                                       //  1 ... 2

      if (step + stepx >= Nsteps) {
         step = Nsteps;
         R = 1.0;
      }

      cx1 = ss_ww * (0.5 + 0.5 * R);                                       //  corners of expanding trapezoid
      cy1 = ss_hh * (0.3 - 0.3 * R);
      cx2 = ss_ww * (0.5 - 0.5 * R);
      cy2 = 0;
      cy3 = ss_hh;
      cy4 = ss_hh - cy1;

      memset(pixels3,0,ss_hh * rsmix);

      for (ii = 0; ii < NWT; ii++) {                                       //  start worker threads
         tbusy[ii] = 1;
         start_detached_thread(ss_rotate_thread2,&Nval[ii]);
      }

      for (ii = 0; ii < NWT; ii++)
         while(tbusy[ii]) zsleep(0.001);

      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pxbmix,0,0);
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);
      gtk_main_iteration_do(0);
   }

   g_object_unref(pxbmix);
   resource_unlock(Fpaintlock);
   gdk_window_thaw_updates(gdkwin);
   return;
}

void * ss_rotate_thread1(void *arg)
{
   using namespace ss_rotate_names;

   int      index = *((int *) (arg));
   int      px, py, ylo, yhi, vpx, vpy;
   uint8    *pix1, *pix3;
   float    Rx, Ry;

   for (px = cx1 + index; px < cx2; px += NWT)
   {
      Rx = 1.0 * (px - cx1) / (cx2 - cx1);
      ylo = cy1 + Rx * (cy2 - cy1);
      yhi = cy4 + Rx * (cy3 - cy4);

      for (py = ylo; py < yhi; py++)
      {
         Ry = 1.0 * (py - ylo) / (yhi - ylo);
         vpx = Rx * (ss_ww - 1);
         vpy = Ry * (ss_hh - 1);

         pix1 = pixels1 + vpy * ss_rsold + vpx * 3;
         pix3 = pixels3 + py * rsmix + px * 3;
         memcpy(pix3,pix1,3);                                              //  v.14.11
      }
   }

   tbusy[index] = 0;
   pthread_exit(0);
   return 0;
}

void * ss_rotate_thread2(void *arg)
{
   using namespace ss_rotate_names;

   int      index = *((int *) (arg));
   int      px, py, ylo, yhi, vpx, vpy;
   uint8    *pix2, *pix3;
   float    Rx, Ry;

   for (px = cx2 + index; px < cx1; px += NWT)
   {
      Rx = 1.0 * (px - cx2) / (cx1 - cx2);
      ylo = cy2 + Rx * (cy1 - cy2);
      yhi = cy3 + Rx * (cy4 - cy3);

      for (py = ylo; py < yhi; py++)
      {
         Ry = 1.0 * (py - ylo) / (yhi - ylo);
         vpx = Rx * (ss_ww - 1);
         vpy = Ry * (ss_hh - 1);

         pix2 = pixels2 + vpy * ss_rsnew + vpx * 3;
         pix3 = pixels3 + py * rsmix + px * 3;
         memcpy(pix3,pix2,3);                                              //  v.14.11
      }
   }

   tbusy[index] = 0;
   pthread_exit(0);
   return 0;
}


//  slowly zoom-in on the image (Ken Burns effect)

void ss_zoomin()                                                           //  v.15.01
{
   float       cx = 0.01 * ss_zoomlocx;                                    //  zoom target, 0.5/0.5 = image midpoint
   float       cy = 0.01 * ss_zoomlocy;
   float       nn, Nsteps = ss_zoomsteps;

   PIXBUF      *pxb1, *pxb2, *pxb3;
   GError      *gerror = 0;
   float       Rm, Rw, dR;
   int         ww1, hh1, ww2, hh2, ww3, hh3;
   int         px2, py2, px3, py3;
   int         px4, py4, px5, py5;
   int         ox1, oy1, ox2, oy2;

   pxb1 = gdk_pixbuf_new_from_file(ss_newfile,&gerror);
   if (! pxb1) {
      zmessageACK(Mwin,0,gerror->message);
      return;
   }

   ww1 = gdk_pixbuf_get_width(pxb1);                                       //  image dimensions, 1x
   hh1 = gdk_pixbuf_get_height(pxb1);

   ww2 = ss_ww * ss_zoomsize;                                              //  scale for final zoom-in at 1x
   hh2 = ss_hh * ss_zoomsize;

   Rm = 1.0 * ww1 / hh1;                                                   //  image width/height ratio
   Rw = 1.0 * ww2 / hh2;                                                   //  window width/height ratio
   dR = fabsf(Rm - Rw) / Rw;                                               //  difference ratio

   if (dR <= 0.01 * ss_cliplimit) {                                        //  difference is within user limit
      if (Rw >= Rm) {
         ww1 = ww2;                                                        //  width fits window
         hh1 = ww1 / Rm;                                                   //  height will be cut
      }
      else {
         hh1 = hh2;                                                        //  height fits window
         ww1 = hh1 * Rm;                                                   //  width will be cut
      }
   }
   else {                                                                  //  difference is too great
      if (Rw >= Rm) {
         hh1 = hh2;                                                        //  height fits window
         ww1 = hh1 * Rm;                                                   //  width will get margins
      }
      else {
         ww1 = ww2;                                                        //  width fits window
         hh1 = ww1 / Rm;                                                   //  height will get margins
      }
   }

   cx = cx * ww1;                                                          //  zoom center target position
   cy = cy * hh1;

   pxb2 = gdk_pixbuf_scale_simple(pxb1,ww1,hh1,BILINEAR);                  //  rescaled image
   g_object_unref(pxb1);
   pxb1 = pxb2;

   ox1 = (ww1 - ww2) / 2;                                                  //  top left corner of image to copy from
   if (ox1 < 0) ox1 = 0;
   oy1 = (hh1 - hh2) / 2;
   if (oy1 < 0) oy1 = 0;

   ox2 = (ww2 - ww1) / 2;                                                  //  top left corner of image to copy to
   if (ox2 < 0) ox2 = 0;
   oy2 = (hh2 - hh1) / 2;
   if (oy2 < 0) oy2 = 0;

   if (ww2 < ww1) ww1 = ww2;                                               //  copy width
   if (hh2 < hh1) hh1 = hh2;                                               //  copy height

   pxb2 = gdk_pixbuf_new(GDKRGB,0,8,ww2,hh2);                              //  output pixbuf matching window w/h
   gdk_pixbuf_fill(pxb2,0);                                                //  black margins
   gdk_pixbuf_copy_area(pxb1,ox1,oy1,ww1,hh1,pxb2,ox2,oy2);                //  insert image
   g_object_unref(pxb1);                                                   //  pxb1 = input image, clipped
   pxb1 = pxb2;                                                            //    or with added margins

   ww1 = ww2;                                                              //  input image size = window size
   hh1 = hh2;

   ww2 = ww2 / ss_zoomsize;                                                //  final zoomed image size
   hh2 = hh2 / ss_zoomsize;                                                //  (will expand to window size)

   cx = cx - ox1 + ox2;                                                    //  zoom target center, adjusted
   cy = cy - oy1 + oy2;                                                    //    for clipping and margins

   px2 = cx - ww2/2;                                                       //  upper left corner of final
   py2 = cy - hh2/2;                                                       //    image segment to display

   if (px2 + ww2 > ww1) px2 = ww1 - ww2;                                   //  shift final image segment
   if (px2 < 0) px2 = 0;                                                   //    to stay within image bounds
   if (py2 + hh2 > hh1) py2 = hh1 - hh2;
   if (py2 < 0) py2 = 0;

   px4 = px2 + ww2;                                                        //  lower right corner of
   py4 = py2 + hh2;                                                        //    final image segment

   if (! resource_lock(Fpaintlock)) return;                                //  block Fpaint()                     v.15.02
   gdk_window_freeze_updates(gdkwin);

   for (nn = 0; nn < Nsteps; nn++)                                         //  loop reducing image segment
   {
      px3 = px2 * nn / Nsteps;                                             //  px3/py3 from 0/0 to px2/py2
      py3 = py2 * nn / Nsteps;
      px5 = ww1 - (ww1 - px4) * nn / Nsteps;                               //  px5/py5 from ww1/hh1 to px4/py4
      py5 = hh1 - (hh1 - py4) * nn / Nsteps;
      ww3 = px5 - px3;                                                     //  ww3/hh3 from ww1/hh1 to ww2/hh2
      hh3 = py5 - py3;

      pxb2 = gdk_pixbuf_new_subpixbuf(pxb1,px3,py3,ww3,hh3);               //  smaller image segment to display
    //pxb3 = gdk_pixbuf_scale_simple(pxb2,ss_ww,ss_hh,BILINEAR);           //  slow, high quality 37 sec.
    //pxb3 = pixbuf_rescale(pxb2,ss_ww,ss_hh);                             //  slow, high quality 33 sec.
      pxb3 = pixbuf_rescale_fast(pxb2,ss_ww,ss_hh);                        //  fast, med. quality 17 sec.         v.15.02

      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pxb3,0,0);                         //  paint
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);

      gtk_main_iteration_do(0);                                            //  v.15.02

      if (ss_escape) goto done;                                            //  break out
      if (nn + 1 >= Nsteps) goto done;

      g_object_unref(pxb2);
      g_object_unref(pxb3);
      continue;

   done:
      g_object_unref(pxb3);                                                //  redo last image with best quality  v.15.02
      pxb3 = gdk_pixbuf_scale_simple(pxb2,ss_ww,ss_hh,BILINEAR);
      ss_cr = gdk_cairo_create(gdkwin);
      gdk_cairo_set_source_pixbuf(ss_cr,pxb3,0,0);                         //  paint
      cairo_paint(ss_cr);
      cairo_destroy(ss_cr);

      g_object_unref(ss_pxbnew);
      ss_pxbnew = pxb3;                                                    //  retain final zoomed image
      g_object_unref(pxb2);                                                //    for next transition
      g_object_unref(pxb1);
      break;
   }

   resource_unlock(Fpaintlock);
   gdk_window_thaw_updates(gdkwin);
   return;
}


//  change monitor background image at regular interval
//  desktop background image must be set as follows:
//     /home/<user>/.fotoxx/background_file

namespace background_names
{
   int      Frun = 0;                                                      //  flag, run or stop
   int      Findex = 0;                                                    //  current file
   int      Nfiles = 0;                                                    //  file count
   char     **filelist = 0;                                                //  files list (images)
   int      seconds = 10;                                                  //  image display time
}


//  menu function

void m_background(GtkWidget *, cchar *)                                    //  v.15.05
{
   using namespace background_names;
   
   int background_dialog_event(zdialog *zd, cchar *event);
   int background_timerfunc(void *);

   zdialog     *zd;
   int         zstat;
   cchar       *title = ZTX("Cycle Background Image");
   
   F1_help_topic = "background";
   
/***
       __________________________________________________
      | [x] [-] [ ]    Cycle Background Image            |
      |                                                  |
      | [Select] album-name   123 images                 |
      | Seconds [___|-+]                                 |
      |                                                  |
      |                              [Proceed] [Cancel]  |
      |__________________________________________________|

***/

   zd = zdialog_new(title,Mwin,Bproceed,Bcancel,null);                     //  user dialog
   zdialog_add_widget(zd,"hbox","hbss","dialog",0,"space=3");
   zdialog_add_widget(zd,"button","selectalbum","hbss",Bselect,"space=5");
   zdialog_add_widget(zd,"label","albumname","hbss",Bnoselection,"space=5");
   zdialog_add_widget(zd,"label","nfiles","hbss",Bnoimages,"space=5");
   zdialog_add_widget(zd,"hbox","hbsecs","dialog",0,"space=2");
   zdialog_add_widget(zd,"label","labsecs","hbsecs",Bseconds,"space=5");
   zdialog_add_widget(zd,"spin","seconds","hbsecs","2|9999|1|10");
   
   Frun = 0;                                                               //  stop timer if running

   zdialog_run(zd,background_dialog_event,"save");                         //  run dialog
   zstat = zdialog_wait(zd);                                               //  wait for completion
   
   if (zstat != 1) {                                                       //  cancel
      zdialog_free(zd);
      return;
   }
   
   if (! Nfiles) {                                                         //  no album given
      zmessageACK(Mwin,0,ZTX("invalid album"));
      zdialog_free(zd);
      return;
   }
   
   zdialog_free(zd);

   Frun = 1;                                                               //  run continuously
   Findex = 0;                                                             //  first image file
   
   background_timerfunc(0);                                                //  first call, immediate
   g_timeout_add_seconds(seconds,background_timerfunc,0);                  //  call every time interval

   return;
}


//  dialog event function - file chooser to select album

int background_dialog_event(zdialog *zd, cchar *event)
{
   using namespace background_names;

   char     *albumfile, *albumname, *file;
   char     buff[XFCC];
   char     countmess[50];
   int      ii, ftype;
   FILE     *fid;

   if (strmatch(event,"enter")) zd->zstat = 1;                             //  [proceed]

   if (zd->zstat == 1) {                                                   //  [proceed]
      if (Nfiles) return 1;
      zmessageACK(Mwin,0,ZTX("invalid album"));                            //  keep dialog open
      zd->zstat = 0;
      return 1;
   }

   if (zd->zstat) return 1;                                                //  cancel or [x]

   if (strmatch(event,"selectalbum"))                                      //  select an album
   {
      zdialog_stuff(zd,"albumname",Bnoselection);
      zdialog_stuff(zd,"nfiles",Bnoimages);

      albumfile = zgetfile(ZTX("open album"),MWIN,"file",albums_dirk);
      if (! albumfile) return 1;
      albumname = strrchr(albumfile,'/') + 1;                              //  get album name
      zdialog_stuff(zd,"albumname",albumname);                             //  update dialog album data

      if (Nfiles) {
         for (ii = 0; ii < Nfiles; ii++)                                   //  free prior file list if any
            zfree(filelist[ii]);
         zfree(filelist);
      }

      Nfiles = 0;

      fid = fopen(albumfile,"r");                                          //  open album file
      if (! fid) {
         zmessageACK(Mwin,0,ZTX("invalid album"));
         return 1;
      }

      while (true) {                                                       //  read all image file names
         file = fgets_trim(buff,XFCC,fid);
         if (! file) break;
         ftype = image_file_type(file);                                    //  screen out deleted files
         if (ftype != 2) continue;
         Nfiles++;                                                         //  count the files
      }

      fclose(fid);
      
      if (! Nfiles) {
         zmessageACK(Mwin,0,ZTX("invalid album"));
         return 1;
      }

      filelist = (char **) zmalloc(Nfiles * sizeof(char *));

      fid = fopen(albumfile,"r");                                          //  open album file again
      
      ii = 0;

      while (ii < Nfiles) {                                                //  read all image file names
         file = fgets_trim(buff,XFCC,fid);
         if (! file) break;
         ftype = image_file_type(file);                                    //  screen out deleted files
         if (ftype != 2) continue;
         filelist[ii] = zstrdup(file);                                     //  add to file list
         ii++;
      }

      if (ii != Nfiles) zappcrash("ii: %d  Nfiles: %d",ii,Nfiles);

      fclose(fid);
      zfree(albumfile);

      snprintf(countmess,50,ZTX("%d images"),Nfiles);
      zdialog_stuff(zd,"nfiles",countmess);
   }

   if (strmatch(event,"seconds"))                                          //  get display seconds
      zdialog_fetch(zd,"seconds",seconds);

   return 1;
}


//  timer function, called at intervals of "seconds"

int background_timerfunc(void *)
{
   using namespace background_names;

   char     *file;
   int      err;

   if (! Frun) return 0;                                                   //  stop the timer
   if (! Nfiles) return 0;

   if (Findex >= Nfiles) Findex = 0;   
   file = filelist[Findex];
   Findex++;

   err = shell_quiet("cp \"%s\" %s/background_file",file,get_zuserdir()); 
   if (err) {
      Frun = 0;
      return 0;
   }

   return 1;
}







